@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 /**************
@@ -28,15 +28,16 @@
 #include <set>
 #include <vector>
 
+#include <glib.h>
+#include <glib/gi18n.h>
 extern "C" {
   #include <unistd.h>
-  #include <glib/gi18n.h>
 }
 
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/messages.h>
 #include <pan/general/time-elapsed.h>
 
@@ -80,7 +81,15 @@ namespace
   }
 }
 
+// detect std::lib
+#include <ciso646>
+#ifdef _LIBCPP_VERSION
+// using libc++
+#include <algorithm>
+#else
+// using libstdc++
 #include <ext/algorithm>
+#endif
 
 void
 DataImpl :: load_newsrc (const Quark       & server,
@@ -115,7 +124,7 @@ DataImpl :: load_newsrc (const Quark       & server,
 
       if (subscribed)
         tmp_sub.push_back (group);
-      else 
+      else
         tmp_unsub.push_back (group);
 
       if (!numbers.empty())
@@ -165,7 +174,7 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
 
   foreach_const (servers_t, _servers, sit) {
     const Quark key (sit->first);
-    const std::string filename (sit->second.newsrc_filename);
+    const std::string filename = file::absolute_fn("", sit->second.newsrc_filename);
     if (file::file_exists (filename.c_str())) {
       LineReader * in (data_io.read_file (filename));
       load_newsrc (key, in, s, u);
@@ -174,8 +183,8 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
   }
 
   // remove duplicates
-  s.erase (std::unique(s.begin(), s.end()), s.end()); 
-  u.erase (std::unique(u.begin(), u.end()), u.end()); 
+  s.erase (std::unique(s.begin(), s.end()), s.end());
+  u.erase (std::unique(u.begin(), u.end()), u.end());
 
   // unsub -= sub
   AlphabeticalQuarkOrdering o;
@@ -193,11 +202,16 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
 void
 DataImpl :: save_newsrc_files (DataIO& data_io) const
 {
+  if (newsrc_autosave_id) {
+    g_source_remove( newsrc_autosave_id );
+    newsrc_autosave_id = 0;
+  }
+
   if (_unit_test)
     return;
 
   // overly-complex optimization: both sit->second.groups and _subscribed
-  // are both ordered by AlphabeticalQuarkOrdering.  
+  // are both ordered by AlphabeticalQuarkOrdering.
   // Where N==sit->second.groups.size() and M==_subscribed.size(),
   // "bool subscribed = _subscribed.count (group)" is N*log(M),
   // but a sorted set comparison is M+N comparisons.
@@ -209,7 +223,8 @@ DataImpl :: save_newsrc_files (DataIO& data_io) const
     const Quark& server (sit->first);
 
     // write this server's newsrc
-    const std::string& filename (sit->second.newsrc_filename);
+    const std::string filename = file::absolute_fn("", sit->second.newsrc_filename);
+
     std::ostream& out (*data_io.write_file (filename));
     std::string newsrc_string;
     alpha_groups_t::const_iterator sub_it (_subscribed.begin());
@@ -272,11 +287,11 @@ DataImpl :: load_group_permissions (const DataIO& data_io)
   }
 
   std::sort (m.begin(), m.end());
-  m.erase (std::unique(m.begin(), m.end()), m.end()); 
+  m.erase (std::unique(m.begin(), m.end()), m.end());
   _moderated.get_container().swap (m);
 
   std::sort (n.begin(), n.end());
-  n.erase (std::unique(n.begin(), n.end()), n.end()); 
+  n.erase (std::unique(n.begin(), n.end()), n.end());
   _nopost.get_container().swap (n);
 
   delete in;
@@ -294,7 +309,7 @@ DataImpl :: save_group_permissions (DataIO& data_io) const
   tmp_t tmp;
   foreach_const (groups_t, _moderated, it) tmp[*it] = 'm';
   foreach_const (groups_t, _nopost, it) tmp[*it] = 'n';
-  
+
   out << "# Permissions: y means posting ok; n means posting not okay; m means moderated.\n"
       << "# Since almost all groups allow posting, Pan assumes that as the default.\n"
       << "# Only moderated or no-posting groups are listed here.\n";
@@ -351,7 +366,7 @@ DataImpl :: load_group_xovers (const DataIO& data_io)
       line.trim();
       if (line.empty() || *line.str=='#')
         continue;
-     
+
       if (line.pop_token(groupname) && line.pop_token(total) && line.pop_token(unread))
       {
         ReadGroup& g (_read_groups[groupname]);
@@ -360,7 +375,7 @@ DataImpl :: load_group_xovers (const DataIO& data_io)
 
         while (line.pop_token (xover))
           if (xover.pop_token(servername,':'))
-            g[servername]._xover_high = strtoul (xover.str, NULL, 10);
+            g[servername]._xover_high = g_ascii_strtoull (xover.str, NULL, 10);
       }
     }
   }
@@ -457,21 +472,21 @@ DataImpl :: save_group_xovers (DataIO& data_io) const
 *****
 ****/
 
-unsigned long
+uint64_t
 DataImpl :: get_xover_high (const Quark  & groupname,
                             const Quark  & servername) const
 {
-  unsigned long high (0ul);
+  uint64_t high (0ul);
   const ReadGroup::Server * rgs (find_read_group_server (groupname, servername));
   if (rgs)
     high = rgs->_xover_high;
   return high;
 }
-                                                                                
+
 void
 DataImpl :: set_xover_high (const Quark & group,
                             const Quark & server,
-                            const unsigned long high)
+                            const uint64_t high)
 {
   //std::cerr << LINE_ID << "setting " << get_server_address(server) << ':' << group << " xover high to " << high << std::endl;
   ReadGroup::Server& rgs (_read_groups[group][server]);
@@ -502,7 +517,7 @@ DataImpl :: add_groups (const Quark       & server,
     std::set_union (s->groups.begin(), s->groups.end(),
                     groups.begin(), groups.end(),
                     std::back_inserter (tmp), o);
-    tmp.erase (std::unique(tmp.begin(), tmp.end()), tmp.end()); 
+    tmp.erase (std::unique(tmp.begin(), tmp.end()), tmp.end());
     s->groups.get_container().swap (tmp);
 
     // make a groups_t of groups we didn't already have,
@@ -516,12 +531,12 @@ DataImpl :: add_groups (const Quark       & server,
     std::set_union (groups.begin(), groups.end(),
                     _unsubscribed.begin(), _unsubscribed.end(),
                     std::back_inserter (tmp), o);
-    tmp.erase (std::unique(tmp.begin(), tmp.end()), tmp.end()); 
+    tmp.erase (std::unique(tmp.begin(), tmp.end()), tmp.end());
     _unsubscribed.get_container().swap (tmp);
   }
 
   {
-    // build lists of the groups that should and should not be in _moderated and _nopost.
+    // build lists of the groups that should and should not be in _moderated and _nopost.t
     // this is pretty cumbersome, but since these lists almost never change it's still
     // a worthwhile tradeoff to get the speed/memory wins of a sorted_vector
     groups_t mod, notmod, post, nopost, tmp;
@@ -547,7 +562,7 @@ DataImpl :: add_groups (const Quark       & server,
     tmp.clear ();
     std::set_difference (_nopost.begin(), _nopost.end(), post.begin(), post.end(), inserter (tmp, tmp.begin()));
     _nopost.swap (tmp);
-    // _nopost += nopost 
+    // _nopost += nopost
     tmp.clear ();
     std::set_union (_nopost.begin(), _nopost.end(), nopost.begin(), nopost.end(), inserter (tmp, tmp.begin()));
     _nopost.swap (tmp);
@@ -648,11 +663,11 @@ DataImpl :: server_get_groups (const Quark& servername, quarks_t& addme) const
 void
 DataImpl :: get_subscribed_groups (std::vector<Quark>& setme) const
 {
-  setme.assign (_subscribed.begin(), _subscribed.end()); 
+  setme.assign (_subscribed.begin(), _subscribed.end());
 }
 
 void
 DataImpl :: get_other_groups (std::vector<Quark>& setme) const
 {
-  setme.assign (_unsubscribed.begin(), _unsubscribed.end()); 
+  setme.assign (_unsubscribed.begin(), _unsubscribed.end());
 }
