@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #include <config.h>
@@ -24,10 +24,11 @@
 #include <vector>
 #include <string>
 #include <cmath>
+#include <glib.h>
+#include <glib/gi18n.h>
 extern "C" {
   #include <sys/types.h> // for chmod
   #include <sys/stat.h> // for chmod
-  #include <glib/gi18n.h>
 }
 #include <pan/general/debug.h>
 #include <pan/general/log.h>
@@ -46,10 +47,12 @@ DataImpl :: GroupHeaders :: GroupHeaders ():
   _ref (0),
   _dirty (false)
 {
+
 }
 
 DataImpl :: GroupHeaders :: ~GroupHeaders ()
 {
+
 }
 
 DataImpl :: ArticleNode*
@@ -116,7 +119,7 @@ DataImpl :: GroupHeaders :: remove_articles (const quarks_t& mids)
     (*it)->_article = 0;
   _dirty = true;
 }
-  
+
 const DataImpl :: GroupHeaders*
 DataImpl :: get_group_headers (const Quark& group) const
 {
@@ -176,7 +179,7 @@ DataImpl :: ref_group (const Quark& group)
     load_headers (*_data_io, group);
   }
   ++h->_ref;
-  //std::cerr << LINE_ID << " group " << group << " refcount up to " << h->_ref << std::endl;
+//  std::cerr << LINE_ID << " group " << group << " refcount up to " << h->_ref << std::endl;
 }
 
 void
@@ -186,16 +189,24 @@ DataImpl :: unref_group   (const Quark& group)
   pan_return_if_fail (h != 0);
 
   --h->_ref;
-  //std::cerr << LINE_ID << " group " << group << " refcount down to " << h->_ref << std::endl;
+//  std::cerr << LINE_ID << " group " << group << " refcount down to " << h->_ref << std::endl;
   if (h->_ref == 0)
   {
-    if (h->_dirty)
+//    if (h->_dirty )
       save_headers (*_data_io, group);
     h->_dirty = false;
     free_group_headers_memory (group);
   }
 }
 
+void
+DataImpl :: fire_article_flag_changed (articles_t& a, const Quark& group)
+{
+  GroupHeaders * h (get_group_headers (group));
+  h->_dirty = true;
+  Data::fire_article_flag_changed(a,group);
+}
+
 void
 DataImpl :: find_nodes (const quarks_t           & mids,
                         nodes_t                  & nodes,
@@ -238,7 +249,7 @@ void
 DataImpl :: load_article (const Quark       & group,
                           Article           * article,
                           const StringView  & references)
-              
+
 {
 #if 0
   std::cerr << LINE_ID << " adding article "
@@ -260,7 +271,8 @@ DataImpl :: load_article (const Quark       & group,
     node = h->_nodes[mid] = &h->_node_chunk.back();
     node->_mid = mid;
   }
-  assert (!node->_article);
+  // !!INFO!! : this is bypassed for now, as it causes an abort on local cache corruptions
+  //assert (!node->_article);
   node->_article = article;
   ArticleNode * article_node = node;
 
@@ -370,7 +382,7 @@ DataImpl :: get_references (const Quark& group, const Article& a) const
     s.insert (0, v.str, v.len);
     node = node->parent;
   }
-std::cerr << "article " << a.message_id << " references " << s << std::endl;
+//  std::cerr << "article " << a.message_id << " references " << s << std::endl;
   return s;
 }
 #endif
@@ -433,7 +445,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
     } while (!line.empty() && *line.str=='#');
 
     const int version (atoi (line.str));
-    if (version==1 || version==2)
+    if (version==1 || version==2 || version == 3)
     {
       // build the symbolic server / group lookup table
       in->getline (line);
@@ -462,7 +474,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
 
       Xref::targets_t targets;
       std::vector<Xref::Target>& targets_v (targets.get_container());
-        
+
       // each article in this group...
       unsigned int expire_count (0);
       in->getline (line);
@@ -476,13 +488,26 @@ DataImpl :: load_headers (const DataIO   & data_io,
         StringView s;
         if (!in->getline (s)) // end of file
           break;
+
+        Article& a (h->alloc_new_article());
+
+
+        // flag line
+        a.flag = false;
+        if (version == 3)
+        {
+          a.flag = atoi(s.str) == 1 ? true : false;
+          in->getline(s);
+        }
+
         if (s.empty() || *s.str!='<') // not a message-id...
           continue;
 
-        Article& a (h->alloc_new_article());
-        a.message_id = s;
+        //message id
+        s.ltrim(); a.message_id = s;
 
         // subject line
+
         in->getline (s); s.ltrim(); a.subject = s;
 
         // author line
@@ -510,7 +535,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
           if (tok.pop_token(server_tok,':') && tok.pop_token(group_tok,':')) {
             target_it->server = server_tok;
             target_it->group = group_tok.len==1 ? xref_lookup[(int)*group_tok.str] : Quark(group_tok);
-            target_it->number = atoi (tok.str);
+            target_it->number = g_ascii_strtoull (tok.str, NULL, 10);
             const Server * server (find_server (target_it->server));
             if (server && ((!server->article_expiration_age) || (days_old <= server->article_expiration_age)))
               ++target_it;
@@ -535,6 +560,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
 
         // found parts...
         part_batch.init (a.message_id, total_part_count, found_part_count);
+//        std::cerr<<"article "<<a.message_id<<" "<<total_part_count<<" "<<found_part_count<<std::endl;
         for (int i(0), count(found_part_count); i<count; ++i)
         {
           const bool gotline (in->getline (s));
@@ -613,18 +639,19 @@ namespace
 
   struct QuarkToSymbol
   {
+    char buf[2];
+    QuarkToSymbol() { buf[1] = '\0'; }
+
     typedef Loki::AssocVector < pan::Quark, char > quark_to_symbol_t;
     quark_to_symbol_t _map;
 
-    const char* operator() (const Quark& quark) const
+    const char* operator() (const Quark& quark)
     {
-      static char buf[2];
       quark_to_symbol_t::const_iterator it (_map.find (quark));
       if (it == _map.end())
         return quark.c_str();
 
       buf[0] = it->second;
-      buf[1] = '\0';
       return buf;
     }
 
@@ -713,10 +740,12 @@ DataImpl :: save_headers (DataIO                       & data_io,
            "#       If has-attachments isn't 't' (for true), fields 2 and 3 are omitted.\n"
            "#       If fields 2 and 3 are equal, the article is `complete'.\n"
            "#    8. One line per parts-found-count: part-index message-id byte-count\n"
+           "#\n"
            "#\n";
 
     // lines moved from line 8 to line 7 in 0.115, causing version 2
-    out << "2\t # file format version number\n";
+    // flag added, version 3 (12/2011, imhotep)
+    out << "3\t # file format version number\n";
 
     // xref lookup section
     frequency_t frequency;
@@ -746,8 +775,9 @@ DataImpl :: save_headers (DataIO                       & data_io,
       const Quark& message_id (a->message_id);
       h->build_references_header (a, references);
 
-      // message-id, subject, author
-      out << message_id << "\n\t"
+      // flag, message-id, subject, author
+      out <<a->flag<<"\n"
+          << message_id << "\n\t"
           << a->subject << "\n\t"
           << author_qts(a->author) << "\n\t";
       // references line *IF* the article has a References header
@@ -824,7 +854,7 @@ DataImpl :: save_headers (DataIO& data_io, const Quark& group) const
    const double time_elapsed (timer.get_seconds_elapsed());
    if (success)
       Log::add_info_va (
-   _("Saved %lu parts, %lu articles in \"%s\" in %.1f seconds (%.0f art/sec)"),
+   _("Saved %lu parts, %lu articles in \"%s\" in %.1f seconds (%.0f articles/sec)"),
          part_count,
          article_count,
          group.c_str(),
@@ -837,6 +867,17 @@ DataImpl :: save_headers (DataIO& data_io, const Quark& group) const
 /*******
 ********
 *******/
+namespace
+{
+  /* autosave newsrc files */
+  gboolean nrc_as_cb(gpointer ptr)
+  {
+    DataImpl *data = static_cast<DataImpl*>(ptr);
+    data->save_newsrc_files();
+
+    return FALSE;
+  }
+}
 
 void
 DataImpl :: mark_read (const Article & a, bool read)
@@ -875,6 +916,9 @@ DataImpl :: mark_read (const Article  ** articles,
     fire_group_counts (group, g._unread_count, g._article_count);
     on_articles_changed (group, it->second, false);
   }
+
+  if( !newsrc_autosave_id && newsrc_autosave_timeout )
+    newsrc_autosave_id = g_timeout_add_seconds( newsrc_autosave_timeout * 60, nrc_as_cb, this);
 }
 
 
@@ -906,6 +950,7 @@ DataImpl :: get_article_scores (const Quark         & group,
 void
 DataImpl :: rescore_articles (const Quark& group, const quarks_t mids)
 {
+
   GroupHeaders * gh (get_group_headers (group));
   if (!gh) // group isn't loaded
     return;
@@ -922,6 +967,24 @@ DataImpl :: rescore_articles (const Quark& group, const quarks_t mids)
   }
 }
 
+void
+DataImpl :: rescore_group_articles (const Quark& group)
+{
+
+  GroupHeaders * gh (get_group_headers (group));
+  if (!gh) // group isn't loaded
+    return;
+
+  ArticleFilter::sections_t sections;
+  _scorefile.get_matching_sections (group.to_view(), sections);
+  foreach (nodes_t, gh->_nodes, it) {
+    if (it->second->_article) {
+      Article& a (*(it->second->_article));
+      a.score = _article_filter.score_article (*this, sections, group, a);
+    }
+  }
+}
+
 void
 DataImpl :: rescore ()
 {
@@ -943,7 +1006,9 @@ DataImpl :: rescore ()
     const Quark& group (*git);
     const GroupHeaders * h (get_group_headers (group));
     foreach_const (nodes_t, h->_nodes, nit)
-      mids.insert (mids.end(), nit->first);
+      //only insert mids for nodes with articles
+      if (nit->second->_article)
+        mids.insert (mids.end(), nit->first);
     if (!mids.empty())
       on_articles_changed (group, mids, true);
   }
@@ -970,7 +1035,7 @@ DataImpl :: add_score (const StringView           & section_wildmat,
     std::ofstream o (filename.c_str(), std::ofstream::app|std::ofstream::out);
     o << '\n' << str << '\n';
     o.close ();
-    ::chmod (filename.c_str(), 0600);
+    chmod (filename.c_str(), 0600);
   }
 
   if (do_rescore)
@@ -1003,7 +1068,7 @@ DataImpl :: comment_out_scorefile_line (const StringView    & filename,
   std::ofstream o (f.c_str(), std::ofstream::trunc|std::ofstream::out);
   o << buf;
   o.close ();
-  ::chmod (f.c_str(), 0600);
+  chmod (f.c_str(), 0600);
 
   // rescore
   if (do_rescore)
@@ -1057,6 +1122,7 @@ DataImpl :: group_clear_articles (const Quark& group)
 void
 DataImpl :: delete_articles (const unique_articles_t& articles)
 {
+
   quarks_t all_mids;
 
   // info we need to batch these deletions per group...
@@ -1091,7 +1157,7 @@ DataImpl :: delete_articles (const unique_articles_t& articles)
 
     // remove the articles from our lookup table...
     GroupHeaders * h (get_group_headers (group));
-    if (h) 
+    if (h)
       h->remove_articles (it->second.mids);
   }
 
@@ -1118,6 +1184,7 @@ DataImpl :: on_articles_changed (const Quark& group, const quarks_t& mids, bool
 void
 DataImpl :: on_articles_added (const Quark& group, const quarks_t& mids)
 {
+
   if (!mids.empty())
   {
     Log::add_info_va (_("Added %lu articles to %s."),
@@ -1173,9 +1240,11 @@ DataImpl :: find_closest_ancestor (const ArticleNode             * node,
 
 Data::ArticleTree*
 DataImpl :: group_get_articles (const Quark       & group,
+                                const Quark       & save_path,
                                 const ShowType      show_type,
-                                const FilterInfo  * filter) const
+                                const FilterInfo  * filter,
+                                const RulesInfo   * rules) const
 {
   // cast const away for group_ref()... consider _groups mutable
-  return new MyTree (*const_cast<DataImpl*>(this), group, show_type, filter);
+  return new MyTree (*const_cast<DataImpl*>(this), group, save_path, show_type, filter, rules);
 }
