@@ -13,20 +13,21 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #include <config.h>
 #include <cassert>
-#include <glib/gmessages.h> // for g_assert
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
+#include <pan/general/file-util.h>
+#include <pan/general/macros.h>
 #include <pan/general/quark.h>
+#include <pan/usenet-utils/filter-info.h>
 #include <pan/data/article.h>
-#include <pan/data/filter-info.h>
 #include "article-filter.h"
 #include "data-impl.h"
+#include "memchunk.h"
 
 using namespace pan;
 
@@ -78,11 +79,41 @@ DataImpl :: MyTree :: get_article (const Quark& mid) const
 }
 
 size_t
-DataImpl :: MyTree :: size () const 
+DataImpl :: MyTree :: size () const
 {
   return _nodes.size();
 }
 
+void
+DataImpl :: MyTree :: set_rules (const Data::ShowType    show_type,
+                                 const RulesInfo * rules )
+{
+
+  if (rules) {
+//    std::cerr<<"set rules "<<rules<<std::endl;
+    _rules = *rules;
+  }
+  else
+  {
+//    std::cerr<<"rules clear my_tree\n";
+    _rules.clear ();
+  }
+
+  _show_type = show_type;
+
+  const GroupHeaders * h (_data.get_group_headers (_group));
+  g_assert (h != 0);
+  const_nodes_v candidates;
+  candidates.reserve (h->_nodes.size());
+  foreach_const (nodes_t, h->_nodes, it) {
+    const ArticleNode * node (it->second);
+    if (node->_article)
+      candidates.push_back ((ArticleNode*)node);
+  }
+
+  apply_rules(candidates);
+}
+
 void
 DataImpl :: MyTree :: set_filter (const Data::ShowType    show_type,
                                   const FilterInfo      * criteria)
@@ -90,7 +121,7 @@ DataImpl :: MyTree :: set_filter (const Data::ShowType    show_type,
   // set the filter...
   if (criteria)
     _filter = *criteria;
-  else 
+  else
     _filter.clear ();
   _show_type = show_type;
 
@@ -114,14 +145,21 @@ DataImpl :: MyTree :: set_filter (const Data::ShowType    show_type,
 
 DataImpl :: MyTree :: MyTree (DataImpl              & data_impl,
                               const Quark           & group,
+                              const Quark           & save_path,
                               const Data::ShowType    show_type,
-                              const FilterInfo      * filter):
+                              const FilterInfo      * filter,
+                              const RulesInfo       * rules):
   _group (group),
-  _data (data_impl)
+  _data (data_impl),
+  _save_path(save_path)
 {
+
   _data.ref_group (_group);
   _data._trees.insert (this);
+
   set_filter (show_type, filter);
+  set_rules (show_type, rules);
+
 }
 
 DataImpl :: MyTree :: ~MyTree ()
@@ -132,7 +170,7 @@ DataImpl :: MyTree :: ~MyTree ()
 }
 
 /****
-*****  
+*****
 ****/
 
 struct
@@ -154,6 +192,100 @@ DataImpl :: MyTree :: NodeMidCompare
     { return a->_mid < b; }
 };
 
+void
+DataImpl :: MyTree :: apply_rules (const_nodes_v& candidates)
+{
+
+//  std::cerr<<"apply rules mytree\n";
+
+  NodeMidCompare compare;
+  const_nodes_v pass;
+  pass.reserve (candidates.size());
+
+  // apply the rules to the whole tree
+  foreach (const_nodes_v, candidates, it) {
+    if (!(*it)->_article)
+      continue;
+    if (!_data._rules_filter.test_article (_data, _rules, _group, *(*it)->_article))
+      pass.push_back (*it);
+  }
+
+  //  maybe include threads or subthreads...
+  if (_show_type == Data::SHOW_THREADS)
+  {
+    const_nodes_v passcopy=pass;
+    foreach (const_nodes_v, passcopy, it) {
+      const ArticleNode *n (*it);
+      while ((n = n->_parent))
+        pass.push_back(n);
+    }
+    std::sort (pass.begin(), pass.end(), compare);
+    pass.erase (std::unique (pass.begin(), pass.end()), pass.end());
+    //std::cerr << LINE_ID << " reduces to " << pass.size() << " threads\n";
+  }
+
+  if (_show_type == Data::SHOW_THREADS || _show_type == Data::SHOW_SUBTHREADS)
+  {
+    unique_nodes_t d;
+    foreach (const_nodes_v, pass, it)
+      accumulate_descendants (d, *it);
+    //std::cerr << LINE_ID << " expands into " << d.size() << " articles\n";
+
+    const_nodes_v fail2;
+    pass.clear ();
+    foreach (unique_nodes_t, d, it) {
+      Article * a ((*it)->_article);
+      if (!_data._rules_filter.test_article (_data, _rules, _group, *a))
+        pass.push_back (*it);
+    }
+  }
+  cache_articles(_data._rules_filter._cached);
+  download_articles(_data._rules_filter._downloaded);
+  _data._rules_filter.finalize(_data);
+
+  quarks_t mids;
+  foreach_const (const_nodes_v, pass, it)
+    mids.insert (mids.end(), (*it)->_mid);
+  remove_articles (mids);
+}
+
+void
+DataImpl :: MyTree :: cache_articles (std::set<const Article*> s)
+{
+  Queue* queue (_data.get_queue());
+  Prefs& prefs (_data.get_prefs());
+  const bool action (prefs.get_flag("rules-autocache-mark-read", false));
+
+  Queue::tasks_t tasks;
+  ArticleCache& cache(_data.get_cache());
+  foreach_const (std::set<const Article*>, s, it)
+    if (!_data.is_read(*it))
+      tasks.push_back (new TaskArticle (_data, _data, **it, cache, _data,
+                                        action ? TaskArticle::ACTION_TRUE : TaskArticle::ACTION_FALSE));
+  if (!tasks.empty())
+    queue->add_tasks (tasks, Queue::BOTTOM);
+}
+
+void
+DataImpl :: MyTree :: download_articles (std::set<const Article*> s)
+{
+  Queue* queue (_data.get_queue());
+
+  Queue::tasks_t tasks;
+  ArticleCache& cache(_data.get_cache());
+  Prefs& prefs (_data.get_prefs());
+  const bool action (prefs.get_flag("rules-auto-dl-mark-read", false));
+  const bool always (prefs.get_flag("mark-downloaded-articles-read", false));
+
+  foreach_const (std::set<const Article*>, s, it)
+    if (!_data.is_read(*it))
+      tasks.push_back (new TaskArticle (_data, _data, **it, cache, _data,
+                                        always ? TaskArticle::ALWAYS_MARK : action ? TaskArticle::ACTION_TRUE : TaskArticle::ACTION_FALSE,
+                                        0, TaskArticle::DECODE, _save_path));
+  if (!tasks.empty())
+    queue->add_tasks (tasks, Queue::BOTTOM);
+}
+
 
 // candidates holds GroupHeader's ArticleNodes pointers
 // candidates are sorted by Mid (see NodeMidCompare)
@@ -201,29 +333,33 @@ DataImpl :: MyTree :: apply_filter (const const_nodes_v& candidates)
       accumulate_descendants (d, *it);
     //std::cerr << LINE_ID << " expands into " << d.size() << " articles\n";
 
+    const_nodes_v fail2;
     pass.clear ();
     foreach_const (unique_nodes_t, d, it) {
       const Article * a ((*it)->_article);
       if (a->score > -9999 || _data._article_filter.test_article (_data, _filter, _group, *a))
         pass.push_back (*it); // pass is now sorted by mid because d was too
       else
-        fail.push_back (*it); // fail is now unsorted and may have duplicates
+        fail2.push_back (*it); // fail2 is sorted by mid because d was too
     }
 
-    // fail cleanup: remove duplicates
-    std::sort (fail.begin(), fail.end(), compare);
-    fail.erase (std::unique (fail.begin(), fail.end()), fail.end());
+    // fail cleanup: add fail2 and remove duplicates.
+    // both are sorted by mid, so set_union will do the job
+    const_nodes_v tmp;
+    tmp.reserve (fail.size() + fail2.size());
+    std::set_union (fail.begin(), fail.end(),
+                    fail2.begin(), fail2.end(),
+                    inserter (tmp, tmp.begin()), compare);
+    fail.swap (tmp);
 
     // fail cleanup: remove newly-passing articles
-    const_nodes_v tmp;
-    tmp.reserve (fail.size());
+    tmp.clear ();
     std::set_difference (fail.begin(), fail.end(),
                          pass.begin(), pass.end(),
-                         inserter (tmp, tmp.begin()),
-                         compare);
+                         inserter (tmp, tmp.begin()), compare);
     fail.swap (tmp);
     //std::cerr << LINE_ID << ' ' << pass.size() << " of "
-    //          << (pass.size() + fail.size()) 
+    //          << (pass.size() + fail.size())
     //          << " make it past the show-thread block\n";
   }
 
@@ -280,7 +416,7 @@ DataImpl :: MyTree :: remove_articles (const quarks_t& mids)
     if (n->_article && n->_parent && !n->_parent->_article)
     {
       // start filling out a diff entry
-      ArticleTree::Diffs::Reparent r;
+      Diffs::Reparent r;
       r.message_id = n->_mid;
       r.old_parent = n->_parent->_mid;
 
@@ -296,7 +432,8 @@ DataImpl :: MyTree :: remove_articles (const quarks_t& mids)
       }
       n->_parent = newparent;
 
-      diffs.reparented.push_back (r);
+      diffs.reparented.insert (diffs.reparented.end(),
+                               std::pair<Quark,Diffs::Reparent>(r.message_id, r));
     }
   }
 
@@ -313,7 +450,7 @@ DataImpl :: MyTree :: accumulate_descendants (unique_nodes_t& descendants,
   // if this node has an article and wasn't already in `descendants',
   // then add it and its children.
 
-  if (node->_article && descendants.insert(node).second)
+  if (node->_article && descendants.insert(node).second || !node->_article)
     foreach_const (ArticleNode::children_t, node->_children, it)
       accumulate_descendants (descendants, *it);
 }
@@ -321,16 +458,19 @@ DataImpl :: MyTree :: accumulate_descendants (unique_nodes_t& descendants,
 void
 DataImpl :: MyTree :: articles_changed (const quarks_t& mids, bool do_refilter)
 {
-  if (do_refilter) {
-    // refilter... this may cause articles to be shown or hidden
-    const_nodes_v nodes;
-    _data.find_nodes (mids, _data.get_group_headers(_group)->_nodes, nodes);
+//  std::cerr<<"articles changed\n";
+
+  const_nodes_v nodes;
+  _data.find_nodes (mids, _data.get_group_headers(_group)->_nodes, nodes);
+  apply_rules (nodes);
+
+  if (do_refilter)
     apply_filter (nodes);
-  }
 
   // fire an update event for any of those mids in our tree...
   nodes_v my_nodes;
   _data.find_nodes (mids, _nodes, my_nodes);
+
   if (!my_nodes.empty()) {
     ArticleTree::Diffs diffs;
     foreach_const (nodes_v, my_nodes, it)
@@ -342,9 +482,13 @@ DataImpl :: MyTree :: articles_changed (const quarks_t& mids, bool do_refilter)
 void
 DataImpl :: MyTree :: add_articles (const quarks_t& mids)
 {
+//  std::cerr<<"add articles\n";
+
   const_nodes_v nodes;
   _data.find_nodes (mids, _data.get_group_headers(_group)->_nodes, nodes);
+  apply_rules (nodes);
   apply_filter (nodes);
+
 }
 
 
@@ -359,13 +503,15 @@ DataImpl :: MyTree :: TwoNodes
 void
 DataImpl :: MyTree :: add_articles (const const_nodes_v& nodes_in)
 {
+//  std::cerr<<"add articles nodes\n";
+
   NodeMidCompare compare;
 
   ///
   ///  1. add the new articles
   ///
 
-  // sort the nodes by Message-Id
+  // sort the nodes by Message-ID
   // so that we can use `nodes' for set operations
   const_nodes_v nodes (nodes_in);
   std::sort (nodes.begin(), nodes.end(), compare);
@@ -384,13 +530,11 @@ DataImpl :: MyTree :: add_articles (const const_nodes_v& nodes_in)
   }
 
   // build new MyTree nodes for each of the articles being added
-  int node_index (_node_chunk.size());
-  _node_chunk.resize (_node_chunk.size() + nodes.size());
   nodes_v tree_nodes;
   tree_nodes.reserve (nodes.size());
   foreach_const (const_nodes_v, nodes, it) {
     const Article * a ((*it)->_article);
-    ArticleNode * node (&_node_chunk[node_index++]);
+    ArticleNode * node (_node_chunk.alloc());
     node->_mid = a->message_id;
     node->_article = const_cast<Article*>(a);
     //std::cerr << LINE_ID << " added " << node->_mid << " to the tree\n";
@@ -430,7 +574,8 @@ DataImpl :: MyTree :: add_articles (const const_nodes_v& nodes_in)
       added.parent = parent->_mid;
     }
 
-    diffs.added.push_back (added);
+    diffs.added.insert (diffs.added.end(),
+                        std::pair<Quark,Diffs::Added>(added.message_id, added));
     //std::cerr << LINE_ID << " child " << added.message_id
     //                     << " has parent " << added.parent << std::endl;
   }
@@ -518,7 +663,8 @@ DataImpl :: MyTree :: add_articles (const const_nodes_v& nodes_in)
     tree_node->_parent = new_parent;
     reparent.message_id = tree_node->_mid;
     reparent.new_parent = new_parent->_mid;
-    diffs.reparented.push_back (reparent);
+    diffs.reparented.insert (diffs.reparented.end(),
+                             std::pair<Quark,Diffs::Reparent>(reparent.message_id, reparent));
     //std::cerr << LINE_ID << " REPARENTED: " << reparent.message_id
     //          << " has a new parent " << reparent.new_parent << std::endl;
   }
