@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #include <config.h>
@@ -26,9 +26,9 @@
 extern "C" {
   #include <sys/types.h> // for chmod
   #include <sys/stat.h> // for chmod
-  #include <glib.h>
-  #include <glib/gi18n.h>
 }
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <pan/general/debug.h>
 #include <pan/general/string-view.h>
 #include <pan/general/file-util.h>
@@ -84,9 +84,9 @@ namespace
           mc.profile_name = *v;
       if (!mc.profile_name.empty())
         mc.profiles[mc.profile_name].clear ();
-    } 
+    }
 
-    if ((element_name == "signature_file") && !mc.profile_name.empty()) {
+    if ((element_name == "signature-file") && !mc.profile_name.empty()) {
       Profile& p (mc.profiles[mc.profile_name]);
       for (const char **k(attribute_names), **v(attribute_vals); *k; ++k, ++v) {
         if (!strcmp(*k,"active"))
@@ -98,6 +98,15 @@ namespace
         }
       }
     }
+    if ((element_name == "gpg-signature") && !mc.profile_name.empty()) {
+      Profile& p (mc.profiles[mc.profile_name]);
+      for (const char **k(attribute_names), **v(attribute_vals); *k; ++k, ++v) {
+        if (!strcmp(*k,"active"))
+          p.use_sigfile = !strcmp (*v, "true");
+        p.sig_type = p.GPGSIG;
+        p.use_gpgsig = true;
+      }
+    }
   }
 
   // Called for close tags </foo>
@@ -113,9 +122,12 @@ namespace
 
     if (!mc.profile_name.empty()) {
       Profile& p (mc.profiles[mc.profile_name]);
-      if (element_name == "signature_file") p.signature_file.assign (t.str, t.len);
+      if (element_name == "signature-file") p.signature_file.assign (t.str, t.len);
+      else if (element_name == "gpg-signature") p.gpg_sig_uid.assign (t.str, t.len);
       else if (element_name == "attribution") p.attribution.assign (t.str, t.len);
       else if (element_name == "fqdn") p.fqdn.assign (t.str, t.len);
+      else if (element_name == "face") p.face.assign (t.str, t.len);
+      else if (element_name == "xface") p.xface.assign (t.str, t.len);
       else if (element_name == "username") p.username.assign (t.str, t.len);
       else if (element_name == "address") p.address.assign (t.str, t.len);
       else if (element_name == "server") p.posting_server = t;
@@ -127,7 +139,7 @@ namespace
 
   void text (GMarkupParseContext *context    UNUSED,
              const gchar         *text,
-             gsize                text_len,  
+             gsize                text_len,
              gpointer             user_data,
              GError             **error      UNUSED)
   {
@@ -186,8 +198,8 @@ void
 ProfilesImpl :: serialize (std::ostream& out) const
 {
   int depth (0);
- 
-  // xml header... 
+
+  // xml header...
   out << "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n";
   out << indent(depth++) << "<posting>\n";
 
@@ -198,17 +210,25 @@ ProfilesImpl :: serialize (std::ostream& out) const
     out << indent(depth) << "<username>" << escaped(it->second.username) << "</username>\n";
     out << indent(depth) << "<address>" << escaped(it->second.address) << "</address>\n";
     out << indent(depth) << "<server>" << escaped(it->second.posting_server.to_view()) << "</server>\n";
-    if (!it->second.signature_file.empty()) {
+    out << indent(depth) << "<face>" << escaped(it->second.face) << "</face>\n";
+    out << indent(depth) << "<xface>" << escaped(it->second.xface) << "</xface>\n";
+    if (!it->second.signature_file.empty() && it->second.sig_type != Profile::GPGSIG) {
       const char * type;
       switch (it->second.sig_type) {
         case Profile::FILE: type = "file"; break;
         case Profile::COMMAND: type = "command"; break;
         default: type = "text"; break;
       }
-      out << indent(depth) << "<signature_file"
+      out << indent(depth) << "<signature-file"
                            << " active=\"" << (it->second.use_sigfile ? "true" : "false") << '"'
                            << " type=\"" << type << '"'
-                           << ">" << escaped(it->second.signature_file) << "</signature_file>\n";
+                           << ">" << escaped(it->second.signature_file) << "</signature-file>\n";
+    }
+    if (it->second.use_gpgsig && !it->second.gpg_sig_uid.empty())
+    {
+      out << indent(depth) << "<gpg-signature"
+                           << " active=\"" << (it->second.use_sigfile ? "true" : "false") << '"'
+                           << ">" << escaped(it->second.gpg_sig_uid) << "</gpg-signature>\n";
     }
     if (!it->second.attribution.empty())
       out << indent(depth) << "<attribution>" << escaped(it->second.attribution) << "</attribution>\n";
@@ -222,7 +242,7 @@ ProfilesImpl :: serialize (std::ostream& out) const
     }
     out << indent(--depth) << "</profile>\n";
   }
-  out << indent(--depth) << "</profiles>\n\n";
+  out << indent(--depth) << "</profiles>\n";
   out << indent(--depth) << "</posting>\n";
 }
 
@@ -293,10 +313,11 @@ ProfilesImpl :: save () const
   std::ofstream out (f.c_str());
   serialize (out);
   out.close ();
-  ::chmod (f.c_str(), 0600);
+  chmod (f.c_str(), 0600);
 }
 
 ProfilesImpl :: ~ProfilesImpl ()
 {
   save ();
+  delete &_data_io;     // we own the DataIO
 }
