@@ -12,7 +12,7 @@ int main (int argc, char *argv[])
     const Quark group (argv[1]);
     for (int i=0; i<8; ++i) {
       DataImpl data;
-      Data::ArticleTree * tree (data.group_get_articles (group));
+      Data::ArticleTree * tree (data.group_get_articles (group,Quark("")));
       delete tree;
     }
   }
