@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #ifndef __Data_h__
@@ -27,15 +27,31 @@
 
 #include <pan/general/macros.h>
 #include <pan/general/quark.h>
+#include <pan/general/compression.h>
 #include <pan/general/string-view.h>
 #include <pan/usenet-utils/scorefile.h>
 #include <pan/data/article.h>
 #include <pan/data/article-cache.h>
+#include <pan/data/encode-cache.h>
+#include <pan/data/cert-store.h>
 #include <pan/data/server-info.h>
 
+#include <pan/gui/prefs.h>
+#include <pan/gui/progress-view.h>
+
+#ifdef HAVE_GKR
+#if !GTK_CHECK_VERSION(3,0,0)
+  #include <gnome-keyring-1/gnome-keyring.h>
+  #include <gnome-keyring-1/gnome-keyring-memory.h>
+#endif /* !GTK_CHECK_VERSION(3,0,0) */
+#endif
+
 namespace pan
 {
   class FilterInfo;
+  class RulesInfo;
+  class Queue;
+  class CertStore;
 
   /**
    * Data Interface class for seeing the mapping between groups and servers.
@@ -64,7 +80,7 @@ namespace pan
    *
    * Also, a read article should change back to unread if it changes
    * from an incomplete multipart to a complete multipart as new
-   * parts are added to it. 
+   * parts are added to it.
    *
    * @ingroup data
    */
@@ -86,11 +102,15 @@ namespace pan
     std::string username;
     std::string address;
     bool use_sigfile;
-    enum { TEXT, FILE, COMMAND };
+    bool use_gpgsig;
+    enum { TEXT, FILE, COMMAND, GPGSIG };
     int sig_type;
+    std::string gpg_sig_uid;
     std::string signature_file;
     std::string attribution;
     std::string fqdn;
+    std::string face;
+    std::string xface;
     Quark posting_server;
 
     void get_from_header (std::string& s) const {
@@ -104,10 +124,11 @@ namespace pan
       return it==headers.end() ? nil : it->second;
     }
 
-    Profile(): use_sigfile(false), sig_type(TEXT) {}
+    Profile(): use_sigfile(false), use_gpgsig(false), sig_type(TEXT) {}
 
     void clear() { username.clear(); address.clear();
                    use_sigfile = false;
+                   use_gpgsig = false;
                    sig_type = TEXT;
                    signature_file.clear(); attribution.clear(); }
   };
@@ -132,7 +153,7 @@ namespace pan
     public:
       virtual void delete_profile (const std::string& profile_name) = 0;
       virtual void add_profile (const std::string& profile_name, const Profile& profile) = 0;
-      
+
     protected:
       Profiles () {}
   };
@@ -157,6 +178,38 @@ namespace pan
     public virtual Profiles,
     public virtual ArticleReferences
   {
+
+    public:
+      struct PasswordData
+      {
+        Quark server;
+        StringView user;
+        gchar* pw;
+      };
+
+    public:
+      struct Server
+      {
+         std::string username;
+         std::string password;
+         std::string host;
+         std::string newsrc_filename;
+         std::string cert;
+         int port;
+         int article_expiration_age;
+         int max_connections;
+         int rank;
+         int ssl_support;
+         int trust;
+         int compression_type;
+         typedef sorted_vector<Quark,true,AlphabeticalQuarkOrdering> groups_t;
+         groups_t groups;
+         gchar* gkr_pw;
+
+         Server(): port(STD_NNTP_PORT), article_expiration_age(31), max_connections(2),
+                    rank(1), ssl_support(0), trust(0), gkr_pw(NULL), compression_type(0) /* NONE */ {}
+      };
+
     protected:
 
       Data () {}
@@ -168,11 +221,31 @@ namespace pan
 
     public:
 
-      virtual ArticleCache& get_cache () = 0; 
+      virtual ArticleCache& get_cache () = 0;
+      virtual const ArticleCache& get_cache () const = 0;
 
-      virtual const ArticleCache& get_cache () const = 0; 
+      virtual EncodeCache& get_encode_cache () = 0;
+      virtual const EncodeCache& get_encode_cache () const = 0;
+
+      virtual CertStore& get_certstore () = 0;
+      virtual const CertStore& get_certstore () const = 0;
 
     public:
+#ifdef HAVE_GKR
+#if GTK_CHECK_VERSION(3,0,0)
+      virtual gboolean password_encrypt (const PasswordData&) = 0;
+      virtual gchar* password_decrypt (PasswordData&) const = 0;
+#else
+      virtual GnomeKeyringResult password_encrypt (const PasswordData&) = 0;
+      virtual GnomeKeyringResult password_decrypt (PasswordData&) const = 0;
+#endif /* GTK_CHECK_VERSION(3,0,0) */
+#endif
+      /** Gets a quark to the provided hostname */
+      virtual bool find_server_by_hn (const std::string& server, Quark& setme) const = 0;
+
+      virtual const Server* find_server (const Quark& server) const = 0;
+
+      virtual Server* find_server (const Quark& server) = 0;
 
       virtual quarks_t get_servers () const = 0;
 
@@ -182,7 +255,11 @@ namespace pan
 
       virtual void set_server_auth (const Quark       & server,
                                     const StringView  & username,
-                                    const StringView  & password) = 0;
+                                    gchar             *&password,
+                                    bool                use_gkr) = 0;
+
+      virtual void set_server_trust (const Quark      & servername,
+                                     const int          setme) = 0;
 
       virtual void set_server_addr (const Quark       & server,
                                     const StringView  & address,
@@ -197,7 +274,14 @@ namespace pan
 
       virtual bool get_server_auth (const Quark   & server,
                                     std::string   & setme_username,
-                                    std::string   & setme_password) const = 0;
+                                    gchar         *& setme_password,
+                                    bool            use_gkr) = 0;
+
+      virtual bool get_server_trust (const Quark  & servername, int&) const = 0;
+
+      virtual bool get_server_compression_type (const Quark  & servername, CompressionType&) const = 0;
+
+      virtual std::string get_server_cert (const Quark & server) const = 0;
 
       virtual int get_server_limits (const Quark & server) const = 0;
 
@@ -208,6 +292,8 @@ namespace pan
 
     public:
 
+      typedef std::vector<const Article*> articles_t;
+
       /**
        * Interface class for objects that listen to a Data's events.
        * @ingroup data
@@ -224,6 +310,13 @@ namespace pan
         virtual void on_group_counts (const Quark& group   UNUSED,
                                       unsigned long unread UNUSED,
                                       unsigned long total  UNUSED) {}
+
+        virtual void on_group_entered (const Quark& group   UNUSED,
+                                      unsigned long unread UNUSED,
+                                      unsigned long total  UNUSED) {}
+
+        /* listener for article flag, don't call too often */
+        virtual void on_article_flag_changed (articles_t& a UNUSED, const Quark& group UNUSED) {}
       };
 
       void add_listener (Listener * l);
@@ -247,6 +340,11 @@ namespace pan
       typedef std::set<Listener*> listeners_t;
       listeners_t _listeners;
 
+    public:
+
+      virtual void fire_article_flag_changed (articles_t& a, const Quark& group);
+      virtual void fire_group_entered (const Quark& group, unsigned long unread, unsigned long total);
+
     /*****************************************************************
     ***
     ***  GROUPS
@@ -354,7 +452,7 @@ namespace pan
            *
            * In the case of new articles, the Tree's existing filter
            * is applied to them and new articles that survive the filter
-           * are threaded into the tree. 
+           * are threaded into the tree.
            *
            * In the case of deleted articles, their children are
            * reparented to the youngest surviving ancestor and then the
@@ -442,7 +540,7 @@ namespace pan
                 (*it++)->on_tree_change (d);
             }
           }
-              
+
 
         /*************************************************************
         ***
@@ -467,14 +565,19 @@ namespace pan
 
           virtual void set_filter (const ShowType     show_type = SHOW_ARTICLES,
                                    const FilterInfo * filter_or_null_to_reset = 0) = 0;
+
+          virtual void set_rules (const ShowType     show_type = SHOW_ARTICLES,
+                                   const RulesInfo * filter_or_null_to_reset = 0) = 0;
       };
 
        /**
         * Get a collection of headers that match the specified filter.
         */
        virtual ArticleTree* group_get_articles (const Quark       & group,
+                                                const Quark       & save_path,  // for auto-download
                                                 const ShowType      show_type = SHOW_ARTICLES,
-                                                const FilterInfo  * criteria = 0) const=0;
+                                                const FilterInfo  * criteria = 0,
+                                                const RulesInfo   * rules    = 0) const=0;
 
        virtual void group_clear_articles (const Quark& group) = 0;
 
@@ -498,9 +601,11 @@ namespace pan
                                                 size_t                end_line,
                                                 bool                  do_rescore) = 0;
 
-        virtual void rescore () = 0;
+       virtual void rescore_articles (const Quark& group, const quarks_t mids) = 0;
 
+       virtual void rescore_group_articles (const Quark& group) = 0;
 
+       virtual void rescore () = 0;
 
     /*****************************************************************
     ***
@@ -542,7 +647,8 @@ namespace pan
                                          const StringView     & references,
                                          const unsigned long    byte_count,
                                          const unsigned long    line_count,
-                                         const StringView     & xref) = 0;
+                                         const StringView     & xref,
+                                         const bool             is_virtual=false) = 0;
 
       /**
        * The last call to xover_unref() for a group can indicate to Data
@@ -553,11 +659,11 @@ namespace pan
 
       /**
        * Returns the high number of the most recent XOVER command
-       * run on the specified {server,group}, or 0 if it's never 
+       * run on the specified {server,group}, or 0 if it's never
        * been run there.
        */
-      virtual unsigned long get_xover_high (const Quark  & group,
-                                    const Quark  & server) const = 0;
+      virtual uint64_t get_xover_high (const Quark  & group,
+                                       const Quark  & server) const = 0;
 
        /**
         * After an XOVER command has been run, its range should be set
@@ -567,7 +673,7 @@ namespace pan
         */
        virtual void set_xover_high (const Quark         & group,
                                     const Quark         & server,
-                                    const unsigned long    high) = 0;
+                                    const uint64_t    high) = 0;
 
        /**
         * Used to fold the Newsrc ranges together:
@@ -576,7 +682,10 @@ namespace pan
         */
        virtual void set_xover_low (const Quark         & group,
                                    const Quark         & server,
-                                   const unsigned long   low) = 0;
+                                   const uint64_t   low) = 0;
+
+      /** Sets the queue interface */
+      virtual void set_queue (Queue* q) = 0;
 
   };
 }
