@@ -1,8 +1,7 @@
 #include <config.h>
 #include <cstdio>
-extern "C"{ 
-  #include <glib/gi18n.h>
-}
+#include <cstdlib>
+#include <glib/gi18n.h>
 #include <cerrno>
 #include "file-util.h"
 #include "line-reader.h"
