@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #ifndef _Quark_h_
@@ -26,21 +26,23 @@
 #include <set>
 #include <string>
 #include <climits>
+#include <string>
 #include <vector>
 
-#if defined(HAVE_EXT_HASH_SET)
+#if defined(HAVE_TR1_UNORDERED_SET)
+# include <tr1/unordered_set>
+#elif defined(HAVE_EXT_HASH_SET)
 # include <ext/hash_set>
 #else
 # include <set>
 #endif
+
 #include <pan/general/string-view.h>
 
 #ifndef UINT32_MAX
 #define UINT32_MAX             (4294967295U)
 #endif
 
-#define get16bits(d) (*((const uint16_t *) (d)))
-
 namespace pan
 {
   /**
@@ -74,12 +76,18 @@ namespace pan
         Impl (): refcount(0), len(0), str(0) {}
         Impl (const StringView& v): refcount(0), len(v.len), str(const_cast<char*>(v.str)) {}
         StringView to_view () const { return StringView(str,len); }
-        bool operator() (const Impl& a, const Impl& b) const { return StringView(str,len) == StringView(b.str,b.len); }
+        //wtf? bool operator() (const Impl& a, const Impl& b) const { return StringView(str,len) == StringView(b.str,b.len); }
         bool operator== (const Impl& b) const { return StringView(str,len) == StringView(b.str,b.len); }
+        bool operator< (const Impl& b) const { return StringView(str,len) < StringView(b.str,b.len); }
       };
 
       struct StringViewHash
       {
+        static uint16_t get16bits( const char * in )
+        {
+          return (in[0]<<8) | in[1];
+        }
+
         /**
          * Paul Hsieh's "SuperFastHash" algorithm, from
          * http://www.azillionmonkeys.com/qed/hash.html
@@ -133,8 +141,10 @@ namespace pan
       };
 
 
-#if defined(HAVE_EXT_HASH_SET)
-      typedef __gnu_cxx::hash_set < Impl, StringViewHash > lookup_t;
+#if defined(HAVE_TR1_UNORDERED_SET)
+      typedef std::tr1::unordered_set<Impl, StringViewHash> lookup_t;
+#elif defined(HAVE_EXT_HASH_SET)
+      typedef __gnu_cxx::hash_set<Impl, StringViewHash> lookup_t;
 #else
       typedef std::set<Impl> lookup_t;
 #endif
@@ -231,7 +241,7 @@ namespace pan
       if (a.empty() && b.empty()) return false;
       if (a.empty()) return true;
       if (b.empty()) return false;
-      return std::strcmp (a.c_str(), b.c_str()) < 0;
+      return ::strcmp (a.c_str(), b.c_str()) < 0;
     }
     bool operator() (const Quark* a, const Quark* b) const {
       return (*this) (*a, *b);
