@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #ifndef __StringView_h__
@@ -22,6 +22,8 @@
 
 #include <iosfwd>
 #include <string>
+#include <cstring>
+#include <glib.h>
 
 namespace pan
 {
@@ -103,6 +105,7 @@ namespace pan
 
          bool operator== (const StringView& p) const { return !strcmp(p); }
          bool operator!= (const StringView& p) const { return !(*this == p); }
+         bool operator<  (const StringView& p) const { return strcmp(p)<0; }
 
          bool operator== (const char *s) const { return !strcmp(str,len,s,strlen(s)); }
          bool operator!= (const char *s) const { return  strcmp(str,len,s,strlen(s)); }
@@ -112,9 +115,9 @@ namespace pan
 
          int strncasecmp (const char * p, unsigned int l) {
            if (len >= l)
-             return ::strncasecmp (str, p, l);
+             return g_ascii_strncasecmp (str, p, l);
            else {
-             int i = ::strncasecmp (str, p, len);
+             int i = g_ascii_strncasecmp (str, p, len);
              if (i)
                return i;
              return -1; // shorter, so less than p
