@@ -13,13 +13,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 #ifndef DOWNLOAD_HEADERS_UI_H
 #define DOWNLOAD_HEADERS_UI_H
 
-#include <gtk/gtkwindow.h>
+#include "gtk-compat.h"
 #include <pan/general/quark.h>
 #include "prefs.h"
 
