@@ -13,18 +13,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #include <config.h>
 #include <cassert>
 #include <deque>
-extern "C" {
-  #include <glib/gi18n.h>
-  #include <gtk/gtk.h>
-}
+#include <glib/gi18n.h>
+#include "gtk-compat.h"
 #include <pan/general/debug.h>
+//#include <pan/general/gdk-threads.h>
 #include <pan/general/log.h>
 #include <pan/general/macros.h>
 #include <pan/general/quark.h>
@@ -33,7 +32,7 @@ extern "C" {
 #include <pan/data/data.h>
 #include "group-pane.h"
 #include "pad.h"
-#include "sexy-icon-entry.h"
+#include "tango-colors.h"
 
 using namespace pan;
 
@@ -59,10 +58,15 @@ namespace
       }
   };
 
+  Quark * virtual_title_quark (0);
   Quark * sub_title_quark (0);
   Quark * other_title_quark (0);
+
   bool is_group (const Quark& name) {
-    return !name.empty() && name!=*sub_title_quark && name!=*other_title_quark;
+    return !name.empty() &&
+        name!=*sub_title_quark &&
+        name!=*other_title_quark &&
+        name!=*virtual_title_quark;
   }
 
   std::string
@@ -215,7 +219,7 @@ namespace
     blah->path = path;
     blah->col = col;
     g_idle_add (maybe_activate_on_idle_idle, blah);
-  } 
+  }
 }
 
 gboolean
@@ -317,6 +321,15 @@ namespace
 
   NoopRowDispose noop_row_dispose;
 
+  namespace
+  {
+  	//Local folders
+  	static const char* folders_groupnames[] = {
+            _("Sent"),
+            _("Drafts")
+          };
+  }
+
   PanTreeStore*
   build_model (const Data         & data,
                const TextMatch    * match,
@@ -328,7 +341,7 @@ namespace
     store->set_row_dispose (&noop_row_dispose);
 
     // find the groups that we'll be adding.
-    std::vector<Quark> groups, sub, unsub;
+    std::vector<Quark> groups, local_folders, sub, unsub;
     data.get_other_groups (groups);
     find_matching_groups (match, groups, unsub);
     groups.clear ();
@@ -339,16 +352,40 @@ namespace
     group_rows.clear ();
     group_rows.reserve (sub.size() + unsub.size());
 
-    MyRow * headers = new MyRow[2];
-    headers[0].groupname = *sub_title_quark;
-    headers[1].groupname = *other_title_quark;
+    MyRow * headers = new MyRow[3];
+    headers[0].groupname = *virtual_title_quark;
+    headers[1].groupname = *sub_title_quark;
+    headers[2].groupname = *other_title_quark;
     g_object_weak_ref (G_OBJECT(store), delete_rows, headers);
- 
-    // 
+
+    //
+    // local folders
+    //
+    MyRow * row = &headers[0];
+    store->append (NULL, row);
+    {
+		const size_t n (G_N_ELEMENTS(folders_groupnames));
+		std::vector<PanTreeStore::Row*> appendme;
+		appendme.reserve (n);
+		MyRow *rows(new MyRow [n]), *r(rows);
+		g_object_weak_ref (G_OBJECT(store), delete_rows, rows);
+
+		unsigned long unused;
+		for (size_t i(0); i!=n; ++i, ++r)
+		{
+			r->groupname = folders_groupnames[i];
+			data.get_group_counts (r->groupname, r->unread, unused);
+			appendme.push_back (r);
+			group_rows.push_back (r);
+		}
+		store->append (row, appendme);
+		expandme.push_back (store->get_iter (row));
+    }
+    //
     //  subscribed
     //
 
-    MyRow * row = &headers[0];
+    row = &headers[1];
     store->append (NULL, row);
     if (!sub.empty())
     {
@@ -373,8 +410,7 @@ namespace
     //
     // unsubscribed
     //
-
-    row = &headers[1];
+    row = &headers[2];
     store->append (NULL, row);
     if (!unsub.empty())
     {
@@ -420,7 +456,7 @@ GroupPane :: on_group_subscribe (const Quark& groupname, bool sub)
   // find out where it should be moved to
   int pos (0);
   GtkTreeIter section_iter, group_iter;
-  if (gtk_tree_model_iter_nth_child (model, &section_iter, NULL, (sub?0:1))) {
+  if (gtk_tree_model_iter_nth_child (model, &section_iter, NULL, (sub?1:2))) {
     if (gtk_tree_model_iter_children (model, &group_iter, &section_iter)) do {
       MyRow * row (dynamic_cast<MyRow*>(_tree_store->get_row (&group_iter)));
       if (groupname.to_string() < row->groupname.c_str())
@@ -428,7 +464,7 @@ GroupPane :: on_group_subscribe (const Quark& groupname, bool sub)
       ++pos;
     } while (gtk_tree_model_iter_next (model, &group_iter));
   }
-  
+
   // move the row
   _tree_store->reparent (_tree_store->get_row(&section_iter), find_row(groupname), pos);
 
@@ -494,10 +530,21 @@ namespace
       gtk_tree_path_free (path);
     }
   }
+  const char * mode_strings [] =
+  {
+    N_("Group"),
+    N_("Group (regex)")
+  };
+
+  enum
+  {
+    _GROUP,
+    _GROUP_REGEX
+  };
 }
 
 void
-GroupPane :: set_filter (const std::string& search_text)
+GroupPane :: set_filter (const std::string& search_text, int mode)
 {
   GtkTreeView * view = GTK_TREE_VIEW(_tree_view);
 
@@ -509,7 +556,10 @@ GroupPane :: set_filter (const std::string& search_text)
   TextMatch match;
   TextMatch * pmatch (0);
   if (!search_text.empty()) {
-    match.set (search_text, TextMatch::CONTAINS, false);
+    if (mode == _GROUP)
+      match.set (search_text, TextMatch::CONTAINS, false);
+    else if (mode == _GROUP_REGEX)
+      match.set (search_text, TextMatch::REGEX, false);
     pmatch = &match;
   }
 
@@ -540,6 +590,7 @@ namespace
   guint activate_soon_tag (0u);
 
   std::string search_text;
+  int mode;
 
  void set_search_entry (GtkWidget * entry, const char * s)
   {
@@ -552,20 +603,30 @@ namespace
                                      GdkEventFocus * ,
                                      gpointer        )
   {
+#if !GTK_CHECK_VERSION(3,0,0)
     gtk_widget_modify_text (w, GTK_STATE_NORMAL, NULL); // resets
+#else
+    gtk_widget_override_color(w, GTK_STATE_FLAG_NORMAL, NULL);
+#endif
     set_search_entry (w, search_text.c_str());
     return false;
   }
 
   void refresh_search_entry (GtkWidget * w)
   {
-    if (search_text.empty() && !GTK_WIDGET_HAS_FOCUS(w))
+    if (search_text.empty() && !gtk_widget_has_focus(w))
     {
+#if !GTK_CHECK_VERSION(3,0,0)
       GdkColor c;
       c.pixel = 0;
-      c.red = c.green = c.blue = 0xAAAA; // light grey
+      c.red = c.green = c.blue = 0xAAAA;
       gtk_widget_modify_text (w, GTK_STATE_NORMAL, &c);
-      set_search_entry (w, _("Group Name"));
+#else
+      GdkRGBA c;
+      gdk_rgba_parse (&c, "0xAAA");
+      gtk_widget_override_color(w, GTK_STATE_FLAG_NORMAL, &c);
+#endif
+      set_search_entry (w, _(mode_strings[mode]));
     }
   }
 
@@ -579,7 +640,7 @@ namespace
 
   void search_activate (GroupPane * pane)
   {
-    pane->set_filter (search_text);
+    pane->set_filter (search_text, mode);
   }
 
   void remove_activate_soon_tag ()
@@ -597,12 +658,21 @@ namespace
     remove_activate_soon_tag ();
   }
 
-  void clear_button_clicked_cb (SexyIconEntry *e, SexyIconEntryPosition, int, gpointer pane_gpointer)
+  void entry_icon_released (GtkEntry*, GtkEntryIconPosition icon_pos, GdkEventButton *, gpointer menu)
   {
-    set_search_entry (GTK_WIDGET(e), "");
-    refresh_search_entry (GTK_WIDGET(e));
-    search_text.clear ();
-    search_entry_activated (NULL, pane_gpointer);
+    if (icon_pos == GTK_ENTRY_ICON_PRIMARY)
+      gtk_menu_popup (GTK_MENU(menu), 0, 0, 0, 0, 0, gtk_get_current_event_time());
+  }
+
+  void clear_button_clicked_cb (GtkEntry * e, GtkEntryIconPosition icon_pos, GdkEventButton *, gpointer pane_gpointer)
+  {
+    if (icon_pos == GTK_ENTRY_ICON_SECONDARY)
+    {
+      set_search_entry (GTK_WIDGET(e), "");
+      refresh_search_entry (GTK_WIDGET(e));
+      search_text.clear ();
+      search_entry_activated (NULL, pane_gpointer);
+    }
   }
 
   gboolean activated_timeout_cb (gpointer pane_gpointer)
@@ -623,6 +693,21 @@ namespace
     search_text = gtk_entry_get_text (GTK_ENTRY(e));
     bump_activate_soon_tag (static_cast<GroupPane*>(pane_gpointer));
   }
+
+  // when the search mode is changed via the menu,
+  // update our state variable and bump the activate timeout.
+  void search_menu_toggled_cb (GtkCheckMenuItem  * menu_item,
+                               gpointer            entry_g)
+  {
+    if (gtk_check_menu_item_get_active  (menu_item))
+    {
+      mode = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(menu_item), "MODE"));
+      refresh_search_entry (GTK_WIDGET(entry_g));
+      GroupPane * h = (GroupPane*) g_object_get_data (
+                                             G_OBJECT(entry_g), "group-pane");
+      bump_activate_soon_tag (h);
+    }
+  }
 }
 
 void
@@ -636,13 +721,17 @@ namespace
 {
   bool shorten;
 
+  std::string def_fg, def_bg;
+
   void
   render_group_name (GtkTreeViewColumn * ,
                      GtkCellRenderer   * renderer,
                      GtkTreeModel      * model,
                      GtkTreeIter       * iter,
-                     gpointer            )
+                     gpointer          gp)
   {
+
+    GroupPane* pane (static_cast<GroupPane*>(gp));
     PanTreeStore * tree (PAN_TREE_STORE(model));
     MyRow * row (dynamic_cast<MyRow*>(tree->get_row (iter)));
     const unsigned long& unread (row->unread);
@@ -657,12 +746,23 @@ namespace
       g_snprintf (buf, sizeof(buf), " (%lu)", unread);
       group_name += buf;
     }
-    //if (unread || total) {
-      //if (unread)
-      //  g_snprintf (buf, sizeof(buf), _(" (%lu of %lu)"), unread, total);
-      //else
+
+    std::string fg_col(pane->get_group_prefs().get_group_color_str(name));
+    if (fg_col.empty())
+    {
+      fg_col = pane->get_prefs().get_color_str_wo_fallback("group-pane-color-fg");
+        if (fg_col.empty())
+          fg_col = def_fg;
+    }
+
+    std::string bg_col (pane->get_prefs().get_color_str_wo_fallback("group-pane-color-bg"));
+    if (bg_col.empty())
+        bg_col = def_bg;
+
     g_object_set (renderer, "text", group_name.c_str(),
                             "weight", (!is_g || unread ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL),
+                            "foreground", fg_col.empty() ? NULL : fg_col.c_str(),
+                            "background", bg_col.empty() ? NULL : bg_col.c_str(),
                             NULL);
   }
 }
@@ -690,9 +790,9 @@ GroupPane :: find_next_subscribed_group (bool unread_only)
   GtkTreeModel * model (gtk_tree_view_get_model (view));
 
   // find how many subscribed groups the tree's got
-  GtkTreeIter sub_iter;
-  gtk_tree_model_iter_nth_child (model, &sub_iter, 0, 0); // 'sub' node
-  const int n_groups = gtk_tree_model_iter_n_children (model, &sub_iter);
+  GtkTreeIter sub_iter, virtual_iter;
+  gtk_tree_model_iter_nth_child (model, &sub_iter, 0, 1); // 'sub' node
+  int n_groups = gtk_tree_model_iter_n_children (model, &sub_iter);
   if (n_groups < 1)
     return 0;
 
@@ -707,7 +807,7 @@ GroupPane :: find_next_subscribed_group (bool unread_only)
     GtkTreePath * path = gtk_tree_model_get_path (model, &sel_iter);
     gint depth = gtk_tree_path_get_depth (path);
     gint* indices = gtk_tree_path_get_indices (path);
-    if (depth==2 && indices[0]==0) // a subscribed group is selected
+    if (depth==2 && indices[0]==1) // a subscribed group is selected
       start_pos = indices[1];
     gtk_tree_path_free (path);
   }
@@ -728,9 +828,28 @@ GroupPane :: find_next_subscribed_group (bool unread_only)
     GtkTreeIter group_iter;
     gtk_tree_model_iter_nth_child (model, &group_iter, &sub_iter, n);
     const MyRow * row (dynamic_cast<MyRow*>(_tree_store->get_row (&group_iter)));
-    if (!unread_only || row->unread)
+    const bool is_virtual (is_virtual_group(row->groupname));
+    if (!is_virtual && (row->unread || !unread_only))
       return gtk_tree_model_get_path (model, &group_iter);
   }
+
+}
+
+void
+GroupPane :: read_group (const StringView& groupname)
+{
+    GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
+    PanTreeStore * tree (PAN_TREE_STORE(gtk_tree_view_get_model(view)));
+    GtkTreeIter iter;
+    const MyRow* row = find_row (groupname);
+    if(!row)
+    {
+      return;
+    }
+    iter = tree->get_iter (row);
+    GtkTreePath *path = gtk_tree_model_get_path(gtk_tree_view_get_model(view), &iter);
+    read_group (path);
+    gtk_tree_path_free (path);
 }
 
 void
@@ -796,23 +915,62 @@ namespace
 GtkWidget*
 GroupPane :: create_filter_entry ()
 {
-  GtkWidget * entry = sexy_icon_entry_new ();
-
-  GtkWidget * icon = gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
-  sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, GTK_IMAGE(icon));
-  sexy_icon_entry_set_icon_highlight(SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, true);
-  gtk_widget_set_size_request (entry, 133, -1);
-
+  GtkWidget * entry = gtk_entry_new ();
+  g_object_set_data (G_OBJECT(entry), "group-pane", this);
+  gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
+                                 GTK_ENTRY_ICON_SECONDARY,
+                                 GTK_STOCK_CLEAR );
+  gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
+                                 GTK_ENTRY_ICON_PRIMARY,
+                                 GTK_STOCK_FIND );
+//  gtk_widget_set_size_request (entry, 133, -1);
   _action_manager.disable_accelerators_when_focused (entry);
   g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), NULL);
   g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), NULL);
   g_signal_connect (entry, "activate", G_CALLBACK(search_entry_activated), this);
-  g_signal_connect (entry, "icon_released", G_CALLBACK(clear_button_clicked_cb), this);
+  g_signal_connect (entry, "icon-release", G_CALLBACK(clear_button_clicked_cb), this);
   entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed_by_user), this);
+
+  GtkWidget * menu = gtk_menu_new ();
+  bool regex = _prefs.get_flag ("use-regex", false);
+  if ( regex == true )
+    mode = 1;
+  else
+    mode = 0;
+  GSList * l = 0;
+  for (int i=0, qty=G_N_ELEMENTS(mode_strings); i<qty; ++i)
+  {
+    GtkWidget * w = gtk_radio_menu_item_new_with_label (l, _(mode_strings[i]));
+    l = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(w));
+    g_object_set_data (G_OBJECT(w), "MODE", GINT_TO_POINTER(i));
+    g_signal_connect (w, "toggled", G_CALLBACK(search_menu_toggled_cb),entry);
+    if (mode == i)
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(w), TRUE);
+    gtk_menu_shell_append (GTK_MENU_SHELL(menu), w);
+    gtk_widget_show (w);
+  }
+  g_signal_connect (entry, "icon-release", G_CALLBACK(entry_icon_released), menu);
+
   refresh_search_entry (entry);
   return entry;
 }
 
+bool
+GroupPane :: is_virtual_group (const Quark& group)
+{
+  bool is_virtual = false;
+
+  for (int i(0); i != G_N_ELEMENTS(folders_groupnames); ++i)
+  {
+          if (group == folders_groupnames[i])
+          {
+                  is_virtual = true;
+                  break;
+          }
+  }
+  return is_virtual;
+}
+
 void
 GroupPane :: on_selection_changed (GtkTreeSelection*, gpointer pane_gpointer)
 {
@@ -825,18 +983,35 @@ GroupPane :: on_selection_changed (GtkTreeSelection*, gpointer pane_gpointer)
     "read-selected-group", "mark-groups-read", "delete-groups-articles",
     "get-new-headers-in-selected-groups", "download-headers"
   };
+  static const char* actions_in_nonvirtual_group[] = {
+    "show-group-preferences-dialog",
+    "subscribe", "unsubscribe",
+    "get-new-headers-in-selected-groups", "download-headers",
+    "refresh-group-list", "get-new-headers-in-subscribed-groups", "post"
+  };
   for (int i=0, n=G_N_ELEMENTS(actions_that_require_a_group); i<n; ++i)
     self->_action_manager.sensitize_action (actions_that_require_a_group[i], have_group);
+
+  // disable some functions for virtual mailbox folder
+  for (int i=0, n=G_N_ELEMENTS(actions_in_nonvirtual_group); i<n; ++i)
+    self->_action_manager.hide_action (actions_in_nonvirtual_group[i], is_virtual_group(group));
 }
 
-GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs):
+GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs, GroupPrefs& group_prefs):
   _prefs (prefs),
+  _group_prefs(group_prefs),
   _data (data),
   _tree_view (0),
   _action_manager (action_manager),
   _dirty_groups_idle_tag (0)
 {
+
+  const PanColors& colors (PanColors::get());
+  def_bg = colors.def_bg;
+  def_fg = colors.def_fg;
+
   shorten = prefs.get_flag ("shorten-group-names", false);
+  virtual_title_quark = new Quark (_("Local Folders"));
   sub_title_quark = new Quark (_("Subscribed Groups"));
   other_title_quark = new Quark (_("Other Groups"));
 
@@ -847,12 +1022,8 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
   g_object_unref (G_OBJECT(_tree_store)); // will die with the view
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW(_tree_view), false);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(_tree_view), false);
-#if GTK_CHECK_VERSION(2,8,0)
   gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW(_tree_view), true);
-#endif
-#if GTK_CHECK_VERSION(2,10,0)
   gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW(_tree_view), true);
-#endif
   expand_iterators (iters, GTK_TREE_MODEL(_tree_store), GTK_TREE_VIEW(_tree_view));
 
   GtkTreeSelection * selection (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
@@ -886,7 +1057,7 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
    gtk_tree_view_append_column (GTK_TREE_VIEW(_tree_view), col);
    gtk_tree_view_set_expander_column (GTK_TREE_VIEW(_tree_view), col);
    gtk_tree_view_column_pack_start (col, text_renderer, true);
-   gtk_tree_view_column_set_cell_data_func (col, text_renderer, render_group_name, 0, 0);
+   gtk_tree_view_column_set_cell_data_func (col, text_renderer, render_group_name, this, 0);
 
   _root = scroll;
   _data.add_listener (this);
@@ -897,6 +1068,7 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
 
 GroupPane :: ~GroupPane ()
 {
+  delete virtual_title_quark;
   delete sub_title_quark;
   delete other_title_quark;
 
@@ -912,11 +1084,11 @@ void
 GroupPane :: refresh_font ()
 {
   if (!_prefs.get_flag ("group-pane-font-enabled", false))
-    gtk_widget_modify_font (_tree_view, 0);
+    gtk_widget_override_font (_tree_view, 0);
   else {
     const std::string str (_prefs.get_string ("group-pane-font", "Sans 10"));
     PangoFontDescription * pfd (pango_font_description_from_string (str.c_str()));
-    gtk_widget_modify_font (_tree_view, pfd);
+    gtk_widget_override_font (_tree_view, pfd);
     pango_font_description_free (pfd);
   }
 }
@@ -934,3 +1106,11 @@ GroupPane :: on_prefs_string_changed (const StringView& key, const StringView&)
   if (key == "group-pane-font")
     refresh_font ();
 }
+
+void
+GroupPane :: on_prefs_color_changed (const StringView& key, const GdkColor&)
+{
+  if (key == "group-color")
+    refresh_dirty_groups ();
+    refresh_font ();
+}
