@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #include <config.h>
@@ -23,17 +23,23 @@
 #include <iostream>
 #include <sstream>
 #include <string>
-extern "C" {
-  #include <glib.h>
-  #include <glib/gi18n.h>
-}
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <pan/general/debug.h>
 #include <pan/general/log.h>
 #include <pan/general/macros.h>
 #include "prefs.h"
+#include "group-prefs.h"
 
 using namespace pan;
 
+Prefs :: Prefs() :
+  _rules_changed(false)
+{}
+
+Prefs :: ~Prefs()
+{}
+
 // called for open tags <foo bar='baz'>
 void
 Prefs :: start_element (GMarkupParseContext *,
@@ -156,6 +162,9 @@ Prefs :: to_string (int depth, std::string& setme) const
   foreach_const (ints_t, _ints, it)
     out << indent(depth) << "<int name='" << escaped(it->first) << "' value='" << it->second << "'/>\n";
 
+  foreach_const (longs_t, _longs, it)
+    out << indent(depth) << "<long name='" << escaped(it->first) << "' value='" << it->second << "'/>\n";
+
   foreach_const (strings_t, _strings, it)
     out << indent(depth) << "<string name='" << escaped(it->first) << "' value='" << escaped(it->second) << "'/>\n";
 
@@ -222,8 +231,8 @@ Prefs :: window_size_allocated_cb (GtkWidget      * widget,
 {
   const char * key ((const char*) g_object_get_data (G_OBJECT(widget), PREFS_WIDGET_KEY));
 
-  const bool maximized = GTK_WIDGET(widget)->window
-                      && (gdk_window_get_state(widget->window) & GDK_WINDOW_STATE_MAXIMIZED);
+  const bool maximized = gtk_widget_get_window(widget)
+                      && (gdk_window_get_state(gtk_widget_get_window(widget)) & GDK_WINDOW_STATE_MAXIMIZED);
   if (!maximized)
   {
     int x(0), y(0);
@@ -273,6 +282,15 @@ Prefs :: get_int (const StringView& key, int fallback) const
   return _ints[key];
 }
 
+int
+Prefs :: get_int_min (const StringView& key, int fallback) const
+{
+  if (!_ints.count (key))
+    _ints[key] = fallback;
+  if (_ints[key] < fallback) _ints[key] = fallback;
+  return _ints[key];
+}
+
 void
 Prefs :: set_int (const StringView& key, int value)
 {
@@ -280,16 +298,38 @@ Prefs :: set_int (const StringView& key, int value)
   fire_int_changed (key, value);
 }
 
+/***
+****  LONG64
+***/
+
+uint64_t
+Prefs :: get_long64 (const StringView& key, uint64_t fallback) const
+{
+  if (!_ints.count (key))
+    _longs[key] = fallback;
+  return _longs[key];
+}
+
+void
+Prefs :: set_long64 (const StringView& key, uint64_t value)
+{
+  _longs[key] = value;
+  fire_long64_changed (key, value);
+}
+
+
 /***
 ****  STRINGS
 ***/
 
-std::string
+StringView
 Prefs :: get_string (const StringView& key, const StringView& fallback) const
 {
+  StringView prefs_string;
   if (!_strings.count (key))
     _strings[key] = fallback;
-  return _strings[key];
+  prefs_string = _strings[key];
+  return prefs_string;
 }
 
 void
@@ -298,6 +338,7 @@ Prefs :: set_string (const StringView& key, const StringView& value)
   std::string& lvalue = _strings[key];
   const std::string old (lvalue);
   lvalue.assign (value.str, value.len);
+
   if (old != lvalue)
     fire_string_changed (key, value);
 }
@@ -350,3 +391,15 @@ Prefs :: get_color_str (const StringView& key, const StringView& fallback) const
 {
   return color_to_string (get_color (key, fallback));
 }
+
+/* get string without fallback option */
+std::string
+Prefs :: get_color_str_wo_fallback (const StringView& key) const
+{
+  std::string res;
+  if (!_colors.count(key)) return "";
+  const GdkColor& col(_colors[key]);
+  return color_to_string (col);
+}
+
+
