@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 /*********************
@@ -23,18 +23,18 @@
 
 #include <config.h>
 #include <iostream>
-extern "C" {
-  #include <glib/gi18n.h>
-  #include <gtk/gtk.h>
-}
+#include <glib/gi18n.h>
+#include "gtk-compat.h"
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
+#include <pan/general/macros.h>
 #include <pan/usenet-utils/gnksa.h>
 #include "hig.h"
 #include "pad.h"
 #include "pan-file-entry.h"
 #include "profiles-dialog.h"
 
+#include <pan/usenet-utils/gpg.h>
+
 using namespace pan;
 
 /***
@@ -43,6 +43,14 @@ using namespace pan;
 
 namespace
 {
+  enum
+  {
+    ROW_FILE,
+    ROW_TEXT,
+    ROW_COMMAND,
+    ROW_GPGSIG
+  };
+
   void set_entry (GtkWidget * w, const std::string& str)
   {
     const char * s = str.empty() ? "" : str.c_str();
@@ -61,6 +69,26 @@ namespace
     gtk_widget_set_sensitive (GTK_WIDGET(sensitize), gtk_toggle_button_get_active(tb));
   }
 
+  void on_signature_type_changed (GtkComboBox* w, gpointer g)
+  {
+    ProfileDialog* d(static_cast<ProfileDialog*>(g));
+    g_return_if_fail(d);
+    gint row = gtk_combo_box_get_active (w);
+
+    if (row == ROW_GPGSIG)
+    {
+      gtk_widget_set_tooltip_text(d->_signature_file_combo_box,
+            _("Please choose your email address according to your PGP key's user id."));
+      gtk_widget_hide (d->_signature_file);
+
+    }
+    else
+    {
+      gtk_widget_set_has_tooltip(d->_signature_file_combo_box,false);
+      gtk_widget_show (d->_signature_file);
+    }
+  }
+
   GtkWidget* make_servers_combo (const Data& data, const Quark sel)
   {
     const quarks_t servers (data.get_servers ());
@@ -77,7 +105,7 @@ namespace
       int port;
       std::string addr;
       data.get_server_addr (server, addr, port);
-  
+
       GtkTreeIter iter;
       gtk_list_store_append (store, &iter);
       gtk_list_store_set (store, &iter, 0, addr.c_str(), 1, server.c_str(), -1);
@@ -92,6 +120,7 @@ namespace
   }
 }
 
+
 ProfileDialog :: ProfileDialog (const Data         & data,
                                 const StringView   & profile_name,
                                 const Profile      & profile,
@@ -102,12 +131,10 @@ ProfileDialog :: ProfileDialog (const Data         & data,
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        GTK_STOCK_APPLY, GTK_RESPONSE_OK,
                                        NULL);
-  gtk_dialog_set_default_response (GTK_DIALOG(_root), GTK_RESPONSE_OK); 
+  gtk_dialog_set_default_response (GTK_DIALOG(_root), GTK_RESPONSE_OK);
   gtk_window_set_role (GTK_WINDOW(_root), "pan-edit-profile-dialog");
 
-  GtkTooltips * ttips = gtk_tooltips_new ();
-  g_object_ref_sink_pan (G_OBJECT(ttips));
-  g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, ttips);
+  GtkWidget* hbox(0), * l(0);
 
   int row (0);
   GtkWidget *t = HIG :: workarea_create ();
@@ -125,6 +152,11 @@ ProfileDialog :: ProfileDialog (const Data         & data,
     HIG :: workarea_add_row (t, &row, _("_Full Name:"), w);
     w = _address_entry = gtk_entry_new ();
     set_entry (w, profile.address);
+#ifdef HAVE_GMIME_CRYPTO
+    gtk_widget_set_tooltip_text(w, _("Your email address.\n"
+                                     "Note that this has to match your PGP signature's address\n"
+                                     "if you want your messages to be PGP-signed or encrypted correctly."));
+#endif
     HIG :: workarea_add_row (t, &row, _("_Email Address:"), w);
     w = _server_combo = make_servers_combo (data, profile.posting_server);
     HIG :: workarea_add_row (t, &row, _("_Post Articles via:"), w);
@@ -140,39 +172,83 @@ ProfileDialog :: ProfileDialog (const Data         & data,
     w = _signature_file = pan:: file_entry_new (_("Signature File"), GTK_FILE_CHOOSER_ACTION_OPEN);
     g_signal_connect (_signature_file_check, "toggled", G_CALLBACK(on_sig_file_toggled), w);
     file_entry_set (w, profile.signature_file.c_str());
-    HIG :: workarea_add_row (t, &row, _("_Signature:"), w);
 
-    GtkListStore * store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
     GtkTreeIter iter;
-    gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter, 0, _("Text File"), 1, Profile::FILE, -1);
-    gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter, 0, _("Text"),      1, Profile::TEXT, -1);
-    gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter, 0, _("Command"),   1, Profile::COMMAND, -1);
-    w = _signature_file_combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL(store));
-    g_signal_connect (_signature_file_check, "toggled", G_CALLBACK(on_sig_file_toggled), w);
-    GtkCellRenderer * renderer (gtk_cell_renderer_text_new ());
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (w), renderer, TRUE);
-    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (w), renderer, "text", 0, NULL);
-    int active = 0;
-    if (profile.sig_type == profile.TEXT) active = 1;
-    if (profile.sig_type == profile.COMMAND) active = 2;
-    gtk_combo_box_set_active (GTK_COMBO_BOX(w), active);
-    HIG :: workarea_add_row (t, &row, _("Signature _Type:"), w);
+    GtkListStore * store;
+    GtkCellRenderer * renderer;
 
+    store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter, 0, _("Text File"),    1, Profile::FILE, -1);
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter, 0, _("Text"),         1, Profile::TEXT, -1);
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter, 0, _("Command"),      1, Profile::COMMAND, -1);
+#ifdef HAVE_GMIME_CRYPTO
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter, 0, _("PGP Signature"),1, Profile::GPGSIG, -1);
+#endif
+    w = gtk_combo_box_new_with_model (GTK_TREE_MODEL(store));
+
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 3);
+    l = gtk_label_new(_("Signature Type: "));
+    gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
+    gtk_box_pack_start(GTK_BOX(hbox), l, false, false, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), w, true, true, 0);
+    _signature_file_combo = hbox;
+    _signature_file_combo_box = w;
+#ifdef HAVE_GMIME_CRYPTO
+    g_signal_connect (w, "changed", G_CALLBACK(on_signature_type_changed), this);
+#endif
+    renderer = gtk_cell_renderer_text_new ();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (w), renderer, true);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (w), renderer, "text", 0, NULL);
+
+    int active = ROW_FILE;
+    if (profile.sig_type == profile.TEXT) active = ROW_TEXT;
+    if (profile.sig_type == profile.COMMAND) active = ROW_COMMAND;
+#ifdef HAVE_GMIME_CRYPTO
+    if (profile.sig_type == profile.GPGSIG) active = ROW_GPGSIG;
+#endif
+
+    gtk_combo_box_set_active (GTK_COMBO_BOX(w), active);
+    GtkWidget* vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 3);
+#if !GTK_CHECK_VERSION(3, 0, 0)
+    gtk_box_set_homogeneous (GTK_BOX(vbox), TRUE);
+#endif
+    gtk_box_pack_start(GTK_BOX(vbox), _signature_file_combo, false, false, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), _signature_file, false, false, 0);
+    HIG :: workarea_add_row (t, &row, "",vbox);
+
+  HIG :: workarea_add_section_divider (t, &row);
+
+  HIG :: workarea_add_section_title (t, &row, _("Avatars"));
+    w = _face_entry = gtk_entry_new ();
+    set_entry (w, profile.face);
+    gtk_widget_set_tooltip_markup (w, _("You can add an avatar icon to your articles with a Base64-encoded PNG.\n"
+                                        "Add the Base64-encoded picture without the trailing <b>“Face:”</b>."));
+    HIG :: workarea_add_row (t, &row, _("_Face:"), w, NULL);
+
+    w = _xface_entry = gtk_entry_new ();
+    set_entry (w, profile.xface);
+    gtk_widget_set_tooltip_markup (w, _("You can add an avatar icon to your articles with a unique X-Face code.\n"
+                                        "Add the code without the trailing <b>\"X-Face:\"</b> \n if it was generated "
+                                        "by a helper program (for example http://www.dairiki.org/xface/xface.php)."));
+    HIG :: workarea_add_row (t, &row, _("_X-Face:"), w, NULL);
   HIG :: workarea_add_section_divider (t, &row);
   HIG :: workarea_add_section_title (t, &row, _("Optional Information"));
     HIG :: workarea_add_section_spacer (t, row, 3);
 
     w = _msgid_fqdn_entry = gtk_entry_new ();
     set_entry (w, profile.fqdn);
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("When posting to Usenet, your article's Message-ID contains a domain name.  You can set a custom domain name here, or leave it blank to let Pan use the domain name from your email address."), "");
+    gtk_widget_set_tooltip_text (w, _("When posting to Usenet, your article's Message-ID contains a domain name.\n"
+                                      "You can set a custom domain name here, or leave it blank to let Pan use the "
+                                      "domain name from your email address."));
     HIG :: workarea_add_row (t, &row, _("Message-ID _Domain Name:"), w, NULL);
 
     w = _attribution_entry = gtk_entry_new ();
     set_entry (w, profile.attribution);
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("%i for Message-ID\n%a for Author and Address\n%n for Author name\n%d for Date"), "");
+    gtk_widget_set_tooltip_text (w, _("%i for Message-ID\n%a for Author and Address\n%n for Author name\n%d for Date"));
     HIG :: workarea_add_row (t, &row, _("_Attribution:"), w, NULL);
 
 
@@ -194,9 +270,12 @@ ProfileDialog :: ProfileDialog (const Data         & data,
     std::string s;
     foreach_const (Profile::headers_t, profile.headers, it)
       s += it->first + ": " + it->second + "\n";
-    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW(w)), s.c_str(), s.size());
+    if (!s.empty())
+      gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW(w)), s.c_str(), s.size());
     GtkWidget * eventbox = gtk_event_box_new ();
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), eventbox, _("Extra headers to be included in your posts, such as\nReply-To: \"Your Name\" <yourname@somewhere.com>\nOrganization: Your Organization"), "");
+    /* Translators: Do not localize Reply-To and the quote marks in \"Your Name\". */
+    gtk_widget_set_tooltip_text (eventbox, _("Extra headers to be included in your articles, such as\nReply-To: \"Your Name\" "
+                                             "<yourname@somewhere.com>\nOrganization: Your Organization\n"));
     GtkWidget * scrolled_window = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scrolled_window),
                                          GTK_SHADOW_IN);
@@ -207,15 +286,19 @@ ProfileDialog :: ProfileDialog (const Data         & data,
     gtk_container_add (GTK_CONTAINER(eventbox), scrolled_window);
     HIG :: workarea_add_row (t, &row, _("E_xtra Headers:"), eventbox, w);
 
-  on_sig_file_toggled (GTK_TOGGLE_BUTTON(_signature_file_check), _signature_file);
-  on_sig_file_toggled (GTK_TOGGLE_BUTTON(_signature_file_check), _signature_file_combo);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(_root)->vbox), t, true, true, 0);
+//  on_sig_file_toggled (GTK_TOGGLE_BUTTON(_signature_file_check), _signature_file);
+//  on_sig_file_toggled (GTK_TOGGLE_BUTTON(_signature_file_check), _signature_file_combo_box);
+
+  gtk_box_pack_start (GTK_BOX( gtk_dialog_get_content_area( GTK_DIALOG(_root))), t, true, true, 0);
   gtk_widget_show_all (t);
 
   if (parent != 0) {
     gtk_window_set_transient_for (GTK_WINDOW(_root), parent);
     gtk_window_set_position (GTK_WINDOW(_root), GTK_WIN_POS_CENTER_ON_PARENT);
   }
+#ifdef HAVE_GMIME_CRYPTO
+  on_signature_type_changed(GTK_COMBO_BOX(_signature_file_combo_box), this);
+#endif
 }
 
 ProfileDialog :: ~ProfileDialog ()
@@ -229,7 +312,8 @@ ProfileDialog :: run_until_valid_or_cancel (ProfileDialog& pd)
   {
     const int response (gtk_dialog_run (GTK_DIALOG(pd.root())));
 
-    if (response == GTK_RESPONSE_CANCEL)
+    /* abort profile creation on cancel and closing of the window */
+    if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT)
       return false;
 
     std::string name;
@@ -262,19 +346,26 @@ ProfileDialog :: get_profile (std::string& profile_name, Profile& profile)
   from_entry (_username_entry, profile.username);
   from_entry (_address_entry, profile.address);
   from_entry (_msgid_fqdn_entry, profile.fqdn);
+  from_entry (_face_entry, profile.face);
+  from_entry (_xface_entry, profile.xface);
   from_entry (_attribution_entry, profile.attribution);
 
   profile.use_sigfile = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(_signature_file_check));
-  from_entry (file_entry_gtk_entry(_signature_file), profile.signature_file);
 
   int type;
   GtkTreeIter iter;
-  GtkComboBox * combo = GTK_COMBO_BOX (_signature_file_combo);
+  GtkComboBox * combo = GTK_COMBO_BOX (_signature_file_combo_box);
   gtk_combo_box_get_active_iter (combo, &iter);
   GtkTreeModel * model (gtk_combo_box_get_model (combo));
   gtk_tree_model_get (model, &iter, 1, &type, -1);
   profile.sig_type = type;
 
+  profile.use_gpgsig = (type == profile.GPGSIG);
+  if (!profile.use_gpgsig)
+    from_entry (file_entry_gtk_entry(_signature_file), profile.signature_file);
+
+  profile.gpg_sig_uid = profile.address;
+
   char * pch;
   combo = GTK_COMBO_BOX (_server_combo);
   gtk_combo_box_get_active_iter (combo, &iter);
@@ -419,14 +510,14 @@ ProfilesDialog :: refresh_buttons ()
 
 namespace
 {
-  void profiles_tree_view_selection_changed_cb (GtkTreeSelection * selection, gpointer user_data)
+  void profiles_tree_view_selection_changed_cb (GtkTreeSelection *, gpointer user_data)
   {
     ProfilesDialog *d = (ProfilesDialog *)user_data;
     d->refresh_buttons ();
   }
-  void tree_view_row_activated_cb (GtkTreeView        * view,
-                                   GtkTreePath        * path,
-                                   GtkTreeViewColumn  * col,
+  void tree_view_row_activated_cb (GtkTreeView        * ,
+                                   GtkTreePath        * ,
+                                   GtkTreeViewColumn  * ,
                                    gpointer             user_data)
   {
     on_edit_button (NULL, user_data);
@@ -449,9 +540,9 @@ ProfilesDialog :: ProfilesDialog (const Data& data, Profiles &profiles, GtkWindo
   //g_signal_connect (GTK_OBJECT(_root), "response", G_CALLBACK(response_cb), this);
 
   // workarea
-  GtkWidget * hbox = gtk_hbox_new (false, PAD);
+  GtkWidget * hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, PAD);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), PAD_BIG);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(_root)->vbox), hbox, true, true, 0);
+  gtk_box_pack_start (GTK_BOX( gtk_dialog_get_content_area( GTK_DIALOG(_root))), hbox, true, true, 0);
 
   // create the list
   GtkWidget * w = _view = gtk_tree_view_new ();
@@ -474,31 +565,24 @@ ProfilesDialog :: ProfilesDialog (const Data& data, Profiles &profiles, GtkWindo
   gtk_widget_set_size_request (w, 300, 300);
 
   // button box
-  GtkTooltips * tips = gtk_tooltips_new ();
-  g_object_ref_sink_pan (G_OBJECT(tips));
-  g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, tips);
-  GtkWidget * bbox = gtk_vbox_new (FALSE, PAD_SMALL);
+  GtkWidget * bbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, PAD_SMALL);
   gtk_box_pack_start (GTK_BOX (hbox), bbox, false, false, 0);
 
   // add button
   w = gtk_button_new_from_stock (GTK_STOCK_ADD);
   gtk_box_pack_start (GTK_BOX (bbox), w, false, false, 0);
-  gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(on_add_button), this);
+  g_signal_connect (w, "clicked", G_CALLBACK(on_add_button), this);
 
   // edit button
-#if GTK_CHECK_VERSION(2,6,0)
   w = gtk_button_new_from_stock (GTK_STOCK_EDIT);
-#else
-  w = gtk_button_new_from_stock (GTK_STOCK_OPEN);
-#endif
   _edit_button = w;
   gtk_box_pack_start (GTK_BOX (bbox), w, false, false, 0);
-  gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(on_edit_button), this);
+  g_signal_connect (w, "clicked", G_CALLBACK(on_edit_button), this);
 
   // remove button
   _remove_button = w = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
   gtk_box_pack_start (GTK_BOX (bbox), w, false, false, 0);
-  gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(on_delete_button), this);
+  g_signal_connect (w, "clicked", G_CALLBACK(on_delete_button), this);
 
   // set sensitive buttons
   refresh_buttons ();
