@@ -17,21 +17,21 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #include <config.h>
 #include <algorithm>
 #include <cassert>
-extern "C" {
 #include <glib/gi18n.h>
-}
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
-#include <pan/general/foreach.h>
+#include <pan/general/utf8-utils.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/usenet-utils/mime-utils.h>
+#include <pan/usenet-utils/text-massager.h>
 #include <pan/data/article-cache.h>
 #include "decoder.h"
 #include "task-article.h"
@@ -49,13 +49,35 @@ namespace
     std::string stripped;
     mime::remove_multipart_from_subject (article.subject.c_str(), stripped);
 
+    const char* str = stripped.c_str();
+    iconv_t c = iconv_open("UTF-8","UTF-8");
+    char * res = __g_mime_iconv_strdup(c, str);
+    iconv_close(c);
+
     char buf[1024];
     if (save)
-      snprintf (buf, sizeof(buf), _("Saving %s"), stripped.c_str());
+      snprintf (buf, sizeof(buf), _("Saving %s"), res);
     else
-      snprintf (buf, sizeof(buf), _("Reading %s"), stripped.c_str());
+      snprintf (buf, sizeof(buf), _("Reading %s"), res);
+
+    g_free(res);
+
     return std::string (buf);
   }
+
+  std::string get_groups_str(const Article& a)
+  {
+    std::string r;
+    quarks_t groups;
+    int cnt(1);
+    foreach_const (Xref, a.xref, xit)
+    {
+      r += xit->group.to_string();
+      if (cnt != a.xref.size() && a.xref.size() != 1) r+=", ";
+      ++cnt;
+    }
+    return r;
+  }
 }
 
 TaskArticle :: TaskArticle (const ServerRank          & server_rank,
@@ -63,20 +85,29 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
                             const Article             & article,
                             ArticleCache              & cache,
                             ArticleRead               & read,
+                            const ArticleActionType& mark_read_action,
                             Progress::Listener        * listener,
                             SaveMode                    save_mode,
-                            const Quark               & save_path):
+                            const Quark               & save_path,
+                            const char                * filename,
+                            const SaveOptions         & options):
   Task (save_path.empty() ? "BODIES" : "SAVE", get_description (article, !save_path.empty())),
-  _save_path (save_path),
+  _save_path (expand_attachment_headers(save_path, article)),
   _server_rank (server_rank),
   _cache (cache),
   _read (read),
+  _mark_read_action (mark_read_action),
   _article (article),
   _time_posted (article.time_posted),
   _save_mode (save_mode),
   _decoder(0),
-  _decoder_has_run (false)
+  _decoder_has_run (false),
+  _groups(get_groups_str(article)),
+  _attachment(filename),
+  _options(options),
+  _paused(false)
 {
+
   cache.reserve (article.get_part_mids());
 
   if (listener != 0)
@@ -119,10 +150,11 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
   // initialize our progress status...
   init_steps (all_bytes);
   set_step (all_bytes - need_bytes);
+  const char *artsub(article.subject.c_str());
   if (save_path.empty())
-    set_status (article.subject.c_str());
+    set_status (artsub);
   else
-    set_status_va (_("Saving %s"), article.subject.c_str());
+    set_status_va (_("Saving %s"), artsub);
 
   update_work ();
 }
@@ -132,7 +164,7 @@ TaskArticle :: ~TaskArticle ()
   // ensure our on_worker_done() doesn't get called after we're dead
   if (_decoder)
       _decoder->cancel_silently();
-  
+
   _cache.release (_article.get_part_mids());
 }
 
@@ -207,7 +239,7 @@ TaskArticle :: use_nntp (NNTP * nntp)
     needed->buf.clear ();
 
     Quark group;
-    unsigned long number (0ul);
+    uint64_t number (0ul);
     needed->xref.find (nntp->_server, group, number);
     if (number)
       nntp->article (group, number, this);
@@ -249,7 +281,7 @@ TaskArticle :: on_nntp_line  (NNTP               * nntp,
 void
 TaskArticle :: on_nntp_done  (NNTP             * nntp,
                               Health             health,
-                              const StringView & response)
+                              const StringView & response UNUSED)
 {
   // find the Needed using this nntp...
   needed_t::iterator it;
@@ -260,8 +292,11 @@ TaskArticle :: on_nntp_done  (NNTP             * nntp,
 
   if (health == OK) { // if download succeeded, save it in the cache
     const StringView view (&it->buf.front(), it->buf.size());
-    if (!_cache.add (it->message_id, view))
-      health = ERR_LOCAL;
+    ArticleCache::CacheResponse res (_cache.add (it->message_id, view));
+    if (ArticleCache::CACHE_OK != res.type)
+      health = res.type == ArticleCache::CACHE_DISK_FULL ? ERR_NOSPACE : ERR_LOCAL;
+      if (health == ERR_NOSPACE)
+        _state.set_health (ERR_NOSPACE);
   }
 
   // std::cerr << LINE_ID << ' ' << it->message_id << " from " << nntp->_server << ": health " << health << std::endl;
@@ -273,6 +308,7 @@ TaskArticle :: on_nntp_done  (NNTP             * nntp,
       break;
 
     case ERR_NETWORK: // if the network is bad...
+    case ERR_NOSPACE: // if there's no space, try again, but pause the queue!
     case ERR_LOCAL: // ...or if we got it but couldn't save it
       it->reset ();
       break;
@@ -291,7 +327,7 @@ TaskArticle :: on_nntp_done  (NNTP             * nntp,
       }
       break;
   }
-  
+
   update_work (nntp);
   check_in (nntp, health);
 }
@@ -310,8 +346,8 @@ TaskArticle :: use_decoder (Decoder* decoder)
   init_steps(100);
   _state.set_working();
   const Article::mid_sequence_t mids (_article.get_part_mids());
-  const ArticleCache :: strings_t filenames (_cache.get_filenames (mids));
-  _decoder->enqueue (this, _save_path, filenames, _save_mode);
+  ArticleCache :: strings_t filenames (_cache.get_filenames (mids));
+  _decoder->enqueue (this, _save_path, filenames, _save_mode, _options, _attachment, _article);
   set_status_va (_("Decoding %s"), _article.subject.c_str());
   debug ("decoder thread was free, enqueued work");
 }
@@ -336,18 +372,38 @@ TaskArticle :: on_worker_done (bool cancelled)
     // now that we're back in the main thread.
 
     foreach_const(Decoder::log_t, _decoder->log_severe, it)
+    {
       Log :: add_err(it->c_str());
+      verbose (it->c_str());
+    }
     foreach_const(Decoder::log_t, _decoder->log_errors, it)
+    {
       Log :: add_err(it->c_str());
+      verbose (it->c_str());
+    }
     foreach_const(Decoder::log_t, _decoder->log_infos, it)
+    {
       Log :: add_info(it->c_str());
+      verbose (it->c_str());
+    }
 
-    if (_decoder->mark_read)
-      _read.mark_read(_article);
+    // marks read if either there was no filter action involved or
+    // the user chose to mark read after the action
+    const bool act_on_action (_mark_read_action == ACTION_TRUE);
+    const bool no_action (_mark_read_action == NO_ACTION);
+    const bool always (_mark_read_action == ALWAYS_MARK);
+    const bool never (_mark_read_action == NEVER_MARK);
+    if (!never)
+    {
+      if (_decoder->mark_read && (no_action || act_on_action || always))
+        _read.mark_read(_article);
+    }
 
     if (!_decoder->log_errors.empty())
       set_error (_decoder->log_errors.front());
 
+    _state.set_health(_decoder->health);
+
     if (!_decoder->log_severe.empty())
       _state.set_health (ERR_LOCAL);
     else {
