@@ -81,10 +81,10 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uulib_id = "$Id: uulib.c,v 1.1.1.1 2006/10/02 20:45:58 csk Exp $";
+char * uulib_id = "$Id$";
 
 #ifdef SYSTEM_WINDLL
-BOOL _export WINAPI 
+BOOL _export WINAPI
 DllEntryPoint (HINSTANCE hInstance, DWORD seginfo,
 	       LPVOID lpCmdLine)
 {
@@ -328,7 +328,7 @@ UUBusyPoll (void)
     }
   }
 #else
-  time_t now;
+  time_t now = time (NULL);
   long msecs;
 
   if (uu_BusyCallback) {
@@ -336,7 +336,6 @@ UUBusyPoll (void)
       msecs = 1;
     }
     else {
-      now   = time(NULL);
       msecs = 1000 * (now - uu_last_secs);
     }
 
@@ -575,7 +574,7 @@ UUstrerror (int code)
  */
 
 int UUEXPORT
-UUSetMsgCallback (void *opaque, 
+UUSetMsgCallback (void *opaque,
 		  void (*func) _ANSI_ARGS_((void *, char *, int)))
 {
   uu_MsgCallback = func;
@@ -656,11 +655,11 @@ UUFNameFilter (char *fname)
 int UUEXPORT
 UULoadFile (char *filename, char *fileid, int delflag)
 {
-  return UULoadFileWithPartNo(filename, fileid, delflag, -1);
+  return UULoadFileWithPartNo(filename, fileid, delflag, -1, NULL);
 }
 
 int UUEXPORT
-UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
+UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, const char *global_subject)
 {
   int res, sr, count=0;
   struct stat finfo;
@@ -668,6 +667,7 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
   uufile *fload;
   itbd *killem;
   FILE *datei;
+  int overridden_subject;
 
   if ((datei = fopen (filename, "rb")) == NULL) {
     UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
@@ -720,7 +720,7 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
     fileid = filename;
 
   while (!feof (datei) && !ferror (datei)) {
-    /* 
+    /*
      * Peek file, or some systems won't detect EOF
      */
     res = fgetc (datei);
@@ -728,7 +728,7 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
       break;
     else
       ungetc (res, datei);
-    
+
     if ((loaded = ScanPart (datei, fileid, &sr)) == NULL) {
       if (sr != UURET_NODATA && sr != UURET_OK && sr != UURET_CONT) {
 	UUkillfread (loaded);
@@ -779,7 +779,24 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
       if (uu_fast_scanning && sr != UURET_CONT) break;
       continue;
     }
-    
+
+    if (partno != -1 && global_subject &&
+        loaded->uudet == YENC_ENCODED && /* only yEnc has a filename in each part */
+        (loaded->subject && loaded->filename &&
+         strstr(loaded->subject, loaded->filename) == NULL)) {
+      /*
+       * Sometimes articles derived from an NZB file have corrupt Subject: lines.
+       * That also makes that UUInsertPartToList() fails to combine files properly.
+       * We detect this situation by checking if the filename from the
+       * part can be found in the Subject: header.
+       */
+      _FP_free(loaded->subject);
+      loaded->subject = _FP_strdup((char *)global_subject);
+      overridden_subject = 1;
+    } else {
+      overridden_subject = 0;
+    }
+
     if ((fload = UUPreProcessPart (loaded, &res)) == NULL) {
       /*
        * no useful data found
@@ -794,6 +811,17 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
       continue;
     }
 
+    if (overridden_subject) {
+      /*
+       * If the yEnc file gives a different file name than the subject,
+       * this is a security risk.
+       *
+       * Override the part file name with the one derived from the global subject.
+       */
+      _FP_free(fload->filename);
+      fload->filename = _FP_strdup(fload->subfname);
+    }
+
     if ((loaded->subject && *(loaded->subject)) ||
 	(loaded->mimeid  && *(loaded->mimeid))  ||
 	(loaded->filename&& *(loaded->filename))||
@@ -809,7 +837,7 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
 		 (loaded->end)      ? "end"   : "",
 		 codenames[loaded->uudet]);
     }
-    
+
     if ((res = UUInsertPartToList (fload))) {
       /*
        * couldn't use the data
@@ -916,7 +944,7 @@ UUDecodeFile (uulist *thefile, char *destname)
 
   /*
    * Determine the name of the target file according to the rules:
-   * 
+   *
    * IF (destname!=NULL) THEN filename=destname;
    * ELSE
    *   filename = thefile->filename
@@ -1069,7 +1097,7 @@ UUInfoFile (uulist *thefile, void *opaque,
    */
 
   if (uu_FileCallback) {
-    if ((res = (*uu_FileCallback) (uu_FileCBArg, 
+    if ((res = (*uu_FileCallback) (uu_FileCBArg,
 				   thefile->thisfile->data->sfname,
 				   uugen_fnbuffer,
 				   1)) != UURET_OK)
@@ -1086,7 +1114,7 @@ UUInfoFile (uulist *thefile, void *opaque,
   else {
     if ((inpfile = fopen (thefile->thisfile->data->sfname, "rb")) == NULL) {
       UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
-		 uustring (S_NOT_OPEN_FILE), 
+		 uustring (S_NOT_OPEN_FILE),
 		 thefile->thisfile->data->sfname,
 		 strerror (uu_errno=errno));
       return UURET_IOERR;
@@ -1101,7 +1129,7 @@ UUInfoFile (uulist *thefile, void *opaque,
   fseek (inpfile, thefile->thisfile->data->startpos, SEEK_SET);
   maxpos = thefile->thisfile->data->startpos + thefile->thisfile->data->length;
 
-  while (!feof (inpfile) && 
+  while (!feof (inpfile) &&
 	 (uu_fast_scanning || ftell(inpfile) < maxpos)) {
     if (_FP_fgets (uugen_inbuffer, 511, inpfile) == NULL)
       break;
@@ -1137,7 +1165,7 @@ UUInfoFile (uulist *thefile, void *opaque,
   fclose (inpfile);
 
   if (uu_FileCallback)
-    (*uu_FileCallback) (uu_FileCBArg, 
+    (*uu_FileCallback) (uu_FileCBArg,
 			thefile->thisfile->data->sfname,
 			uugen_fnbuffer, 0);
 
@@ -1146,7 +1174,7 @@ UUInfoFile (uulist *thefile, void *opaque,
 
   return UURET_OK;
 }
-	    
+
 int UUEXPORT
 UURenameFile (uulist *thefile, char *newname)
 {
