@@ -52,12 +52,13 @@
 #endif
 
 #include <crc32.h>
+#include <zlib.h>
 #include <uudeview.h>
 #include <uuint.h>
 #include <fptools.h>
 #include <uustring.h>
 
-char * uunconc_id = "$Id: uunconc.c 2 2006-10-02 20:45:58Z csk $";
+char * uunconc_id = "$Id$";
 
 /* for braindead systems */
 #ifndef SEEK_SET
@@ -222,7 +223,7 @@ UUBrokenByNetscape (char *string)
  * Try to repair a Netscape-corrupted line of data.
  * This must only be called on corrupted lines, since non-Netscape
  * data may even _get_ corrupted by this procedure.
- * 
+ *
  * Some checks are included multiply to speed up the procedure. For
  * example: (*p1!='<' || strnicmp(p1,"</a>",4)). If the first expression
  * becomes true, the costly function isn't called :-)
@@ -262,7 +263,7 @@ UUNetscapeCollapse (char *string)
   while (*p1) {
     if (*p1 == '<') {
       if ((_FP_strnicmp (p1, "<ahref=", 7) == 0 ||
-	   _FP_strnicmp (p1, "<a href=",8) == 0) && 
+	   _FP_strnicmp (p1, "<a href=",8) == 0) &&
 	  (_FP_strstr (p1, "</a>") != 0 || _FP_strstr (p1, "</A>") != 0)) {
 	while (*p1 && *p1!='>')        p1++;
 	if (*p1=='\0' || *(p1+1)!='<') return 0;
@@ -304,6 +305,9 @@ UUValidData (char *ptr, int encoding, int *bhflag)
     return 0;              /* bad string */
   }
 
+  if (encoding == YENC_ENCODED)
+    return YENC_ENCODED;
+
   while (*s && *s!='\012' && *s!='\015') {
     s++;
     len++;
@@ -634,20 +638,20 @@ UUDecodeLine (char *s, char *d, int method)
 
       if(i-- > 0)
 	d[count++] = c;
-      
+
       cc <<= 4;
       c    = table[ACAST(*s++)];
       cc  |= (c >> 2);
-      
+
       if(i-- > 0)
 	d[count++] = cc;
-      
+
       c <<= 6;
       c |= table[ACAST(*s++)];
-      
+
       if(i-- > 0)
 	d[count++] = c;
-      
+
       j -= 4;
     }
   }
@@ -747,7 +751,7 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
 
   uulboundary = -1;
 
-  while (!feof (datain) && 
+  while (!feof (datain) &&
 	 (ftell(datain)<maxpos || flags&FL_TOEND ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
     if (_FP_fgets (line, 255, datain) == NULL)
@@ -821,7 +825,7 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
      * encapsulation line is conceptually attached to the boundary.
      * So if the part ends here, don't print a line break"
      */
-    if (val && (!feof (datain) && 
+    if (val && (!feof (datain) &&
 		(ftell(datain)<maxpos || flags&FL_TOEND ||
 		 (!(flags&FL_PROPER) && uu_fast_scanning))))
       fprintf (dataout, "%s\n", p1);
@@ -844,7 +848,7 @@ UUDecodePT (FILE *datain, FILE *dataout, int *state,
 
   uulboundary = -1;
 
-  while (!feof (datain) && 
+  while (!feof (datain) &&
 	 (ftell(datain)<maxpos || flags&FL_TOEND ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
     if (_FP_fgets (line, 255, datain) == NULL)
@@ -1005,7 +1009,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
     *state = BEGIN;
   }
 
-  while (!feof (datain) && *state != DONE && 
+  while (!feof (datain) && *state != DONE &&
 	 (ftell(datain)<maxpos || flags&FL_TOEND || maxpos==-1 ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
     if (_FP_fgets (line, line_len, datain) == NULL)
@@ -1035,7 +1039,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       vlc = 0;
       continue;
     }
-    
+
     /*
      * Busy Polls
      */
@@ -1124,7 +1128,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  if ((ptr = _FP_strstr (line, " end=")) == NULL) {
 	    break;
 	  }
-       
+
 	  yepartends = atoi (ptr + 5);
 	}
 	tf = 1;
@@ -1133,7 +1137,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       else {
 	continue;
       }
-      
+
       tc = tf = vlc = 0;
       lc[0] = lc[1] = 0;
     }
@@ -1205,8 +1209,8 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  count  = UUDecodeLine (line, oline, method);
 	  if (method == YENC_ENCODED) {
 	    if (yepartends)
-	      yepartcrc = crc32(yepartcrc, oline, count);
-	    yefilecrc = crc32(yefilecrc, oline, count);
+	      yepartcrc = crc32(yepartcrc, (unsigned char*)oline, count);
+	    yefilecrc = crc32(yefilecrc, (unsigned char*)oline, count);
 	    yepartsize += count;
 	  }
 	  vlc++; lc[1]++;
@@ -1279,7 +1283,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
   if (*state  == DONE ||
       (*state == DATA && method == B64ENCODED &&
        vflag == B64ENCODED && (flags&FL_PROPER || haddh))) {
-    for (tf=0; tf<tc; tf++) 
+    for (tf=0; tf<tc; tf++)
       count += UUDecodeLine (save[tf], oline + count, method);
     if (count) {
       if (method == BH_ENCODED) {
@@ -1326,8 +1330,12 @@ UUDecode (uulist *data)
   if (data->state & UUFILE_NODATA)
     return UURET_NODATA;
 
-  if (data->state & UUFILE_NOBEGIN && !uu_desperate)
-    return UURET_NODATA;
+  if (data->state & UUFILE_NOBEGIN) {
+    if (!uu_desperate)
+      return UURET_NODATA;
+    UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+               uustring(S_DATA_SUSPICIOUS));
+  }
 
   if (data->uudet == PT_ENCODED)
     mode = "wt";	/* open text files in text mode */
@@ -1374,8 +1382,10 @@ UUDecode (uulist *data)
    * it in desperate mode
    */
 
-  if ((data->state & UUFILE_NOBEGIN) && uu_desperate)
+  if ((data->state & UUFILE_NOBEGIN) && uu_desperate) {
+    UUMessage (uunconc_id, __LINE__, UUMSG_WARNING, uustring(S_DATA_SUSPICIOUS));
     state = DATA;
+  }
 
   (void) UUDecodeLine (NULL, NULL, 0);                   /* init */
   (void) UUbhwrite    (NULL, 0, 0, NULL);                /* dito */
@@ -1387,15 +1397,15 @@ UUDecode (uulist *data)
   progress.action = 0;
   if (data->filename != NULL) {
     _FP_strncpy (progress.curfile,
-		 (strlen(data->filename)>255)?
-		 (data->filename+strlen(data->filename)-255):data->filename,
-		 256);
+		 (strlen(data->filename)>2047)?
+		 (data->filename+strlen(data->filename)-2047):data->filename,
+		 2048);
   }
   else {
     _FP_strncpy (progress.curfile,
-		 (strlen(data->binfile)>255)?
-		 (data->binfile+strlen(data->binfile)-255):data->binfile,
-		 256);
+		 (strlen(data->binfile)>2047)?
+		 (data->binfile+strlen(data->binfile)-2047):data->binfile,
+		 2048);
   }
   progress.partno   =  0;
   progress.numparts =  0;
@@ -1408,17 +1418,23 @@ UUDecode (uulist *data)
     progress.numparts = (iter->partno)?iter->partno:1;
     iter = iter->NEXT;
   }
-  
+
   /*
    * let's rock!
    */
 
   iter = data->thisfile;
   while (iter) {
-    if (part != -1 && iter->partno != part+1 && !uu_desperate)
-      break;
-    else
-      part = iter->partno;
+
+    if (part==-1 && iter->partno!=1)
+       UUMessage (uunconc_id, __LINE__, UUMSG_WARNING, "Missing everything before part #%d", iter->partno);
+    if (part!=-1 && iter->partno!=part+1) {
+      if (!uu_desperate)
+        break;
+      UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
+                 uustring(S_PART_MISSING), part);
+    }
+    part = iter->partno;
 
     if (iter->data->sfname == NULL) {
       iter = iter->NEXT;
@@ -1474,7 +1490,7 @@ UUDecode (uulist *data)
     iter = iter->NEXT;
   }
 
-  if (state == DATA && 
+  if (state == DATA &&
       (data->uudet == B64ENCODED || data->uudet == QP_ENCODED ||
        data->uudet == PT_ENCODED))
     state = DONE; /* assume we're done */
@@ -1546,7 +1562,7 @@ UUDecode (uulist *data)
     r[0] = fgetc (datain);
     hb   = (int) r[0] + 22;
     fseek (datain, (int) r[0] + 12, SEEK_SET);
-    fread (r, 1, 8, datain);
+    size_t res = fread (r, 1, 8, datain);
 
     dsize = (((long) 1 << 24) * (long) r[0]) +
             (((long) 1 << 16) * (long) r[1]) +
